@@ -65,25 +65,23 @@ import os
 import re
 import time
 
-import six
-
 from swift.common import constraints
 from swift.common.swob import Range, bytes_to_wsgi, normalize_etag, \
     wsgi_to_str
 from swift.common.utils import json, public, reiterate, md5, Timestamp
-from swift.common.db import utf8encode
 from swift.common.request_helpers import get_container_update_override_key, \
     get_param
 
-from six.moves.urllib.parse import quote, urlparse
+from urllib.parse import quote, urlparse
 
 from swift.common.middleware.s3api.controllers.base import Controller, \
     bucket_operation, object_operation, check_container_existence
 from swift.common.middleware.s3api.s3response import InvalidArgument, \
-    ErrorResponse, MalformedXML, BadDigest, KeyTooLongError, \
-    InvalidPart, BucketAlreadyExists, EntityTooSmall, InvalidPartOrder, \
-    InvalidRequest, HTTPOk, HTTPNoContent, NoSuchKey, NoSuchUpload, \
-    NoSuchBucket, BucketAlreadyOwnedByYou, ServiceUnavailable
+    ErrorResponse, MalformedXML, KeyTooLongError, InvalidPart, \
+    BucketAlreadyExists, EntityTooSmall, InvalidPartOrder, InvalidRequest, \
+    HTTPOk, HTTPNoContent, NoSuchKey, NoSuchUpload, NoSuchBucket, \
+    BucketAlreadyOwnedByYou, ServiceUnavailable, PreconditionFailed, \
+    S3NotImplemented
 from swift.common.middleware.s3api.utils import unique_id, \
     MULTIUPLOAD_SUFFIX, S3Timestamp, sysmeta_header
 from swift.common.middleware.s3api.etree import Element, SubElement, \
@@ -293,8 +291,6 @@ class UploadsController(Controller):
 
             :return (non_delimited_uploads, common_prefixes)
             """
-            if six.PY2:
-                (prefix, delimiter) = utf8encode(prefix, delimiter)
             non_delimited_uploads = []
             common_prefixes = set()
             for upload in uploads:
@@ -363,10 +359,7 @@ class UploadsController(Controller):
                     new_uploads, prefix, delimiter)
             uploads.extend(new_uploads)
             prefixes.extend(new_prefixes)
-            if six.PY2:
-                query['marker'] = objects[-1]['name'].encode('utf-8')
-            else:
-                query['marker'] = objects[-1]['name']
+            query['marker'] = objects[-1]['name']
 
         truncated = len(uploads) >= maxuploads
         if len(uploads) > maxuploads:
@@ -542,10 +535,7 @@ class UploadController(Controller):
             if not new_objects:
                 break
             objects.extend(new_objects)
-            if six.PY2:
-                query['marker'] = new_objects[-1]['name'].encode('utf-8')
-            else:
-                query['marker'] = new_objects[-1]['name']
+            query['marker'] = new_objects[-1]['name']
 
         last_part = 0
 
@@ -642,10 +632,7 @@ class UploadController(Controller):
                 container = req.container_name + MULTIUPLOAD_SUFFIX
                 obj = bytes_to_wsgi(o['name'].encode('utf-8'))
                 req.get_response(self.app, container=container, obj=obj)
-            if six.PY2:
-                query['marker'] = objects[-1]['name'].encode('utf-8')
-            else:
-                query['marker'] = objects[-1]['name']
+            query['marker'] = objects[-1]['name']
             resp = req.get_response(self.app, 'GET', container, '',
                                     query=query)
             objects = json.loads(resp.body)
@@ -660,6 +647,14 @@ class UploadController(Controller):
         Handles Complete Multipart Upload.
         """
         upload_id = get_param(req, 'uploadId')
+        # Check for conditional requests before getting upload info so the
+        # headers can't bleed into the HEAD
+        if req.headers.get('If-None-Match', '*') != '*' or any(
+                h in req.headers for h in (
+                    'If-Match', 'If-Modified-Since', 'If-Unmodified-Since')):
+            raise S3NotImplemented(
+                'Conditional uploads are not supported.')
+
         resp, is_marker = _get_upload_info(req, self.app, upload_id)
         if (is_marker and
                 resp.sw_headers.get('X-Backend-Timestamp') >= Timestamp.now()):
@@ -702,16 +697,12 @@ class UploadController(Controller):
             xml = req.xml(MAX_COMPLETE_UPLOAD_BODY_SIZE)
             if not xml:
                 raise InvalidRequest(msg='You must specify at least one part')
-            if 'content-md5' in req.headers:
-                # If an MD5 was provided, we need to verify it.
-                # Note that S3Request already took care of translating to ETag
-                if req.headers['etag'] != md5(
-                        xml, usedforsecurity=False).hexdigest():
-                    raise BadDigest(content_md5=req.headers['content-md5'])
+            # If an MD5 was provided, we need to verify it.
+            if req.check_md5(xml):
                 # We're only interested in the body here, in the
-                # multipart-upload controller -- *don't* let it get
+                # multipart-upload controller -- *don't* let etag get
                 # plumbed down to the object-server
-                del req.headers['etag']
+                req.headers.pop('etag', None)
 
             complete_elem = fromstring(
                 xml, 'CompleteMultipartUpload', self.logger)
@@ -749,13 +740,17 @@ class UploadController(Controller):
 
         s3_etag = '%s-%d' % (s3_etag_hasher.hexdigest(), len(manifest))
         s3_etag_header = sysmeta_header('object', 'etag')
-        if resp.sysmeta_headers.get(s3_etag_header) == s3_etag:
-            # This header should only already be present if the upload marker
-            # has been cleaned up and the current target uses the same
-            # upload-id; assuming the segments to use haven't changed, the work
-            # is already done
+        # This header should only already be present if the upload marker
+        # has been cleaned up and the current target uses the same upload-id
+        already_uploaded_s3_etag = resp.sysmeta_headers.get(s3_etag_header)
+        if already_uploaded_s3_etag == s3_etag:
+            # If the segments to use haven't changed, the work is already done
             return HTTPOk(body=_make_complete_body(req, s3_etag, False),
                           content_type='application/xml')
+        elif already_uploaded_s3_etag:
+            # If the header's present but *doesn't* match, upload-id is
+            # no longer valid
+            raise NoSuchUpload(upload_id=upload_id)
         headers[s3_etag_header] = s3_etag
         # Leave base header value blank; SLO will populate
         c_etag = '; s3_etag=%s' % s3_etag
@@ -810,7 +805,10 @@ class UploadController(Controller):
                                 continue
                             body.append(chunk)
                         body = json.loads(b''.join(body))
-                        if body['Response Status'] != '201 Created':
+                        if body['Response Status'] == \
+                                '412 Precondition Failed':
+                            raise PreconditionFailed
+                        elif body['Response Status'] != '201 Created':
                             for seg, err in body['Errors']:
                                 if err == too_small_message:
                                     raise EntityTooSmall()
