@@ -270,11 +270,27 @@ A GET request with the query parameters::
 will return the contents of the original manifest as it was sent by the client.
 The main purpose for both calls is solely debugging.
 
-When the manifest object is uploaded you are more or less guaranteed that
-every segment in the manifest exists and matched the specifications.
-However, there is nothing that prevents the user from breaking the
-SLO download by deleting/replacing a segment referenced in the manifest. It is
-left to the user to use caution in handling the segments.
+A GET request to a manifest object with the query parameter::
+
+    ?part-number=<n>
+
+will return the contents of the ``nth`` segment. Segments are indexed from 1,
+so ``n`` must be an integer between 1 and the total number of segments in the
+manifest. The response status will be ``206 Partial Content`` and its headers
+will include: an ``X-Parts-Count`` header equal to the total number of
+segments; a ``Content-Length`` header equal to the length of the specified
+segment; a ``Content-Range`` header describing the byte range of the specified
+part within the SLO. A HEAD request with a ``part-number`` parameter will also
+return a response with status ``206 Partial Content`` and the same headers.
+
+.. note::
+
+    When the manifest object is uploaded you are more or less guaranteed that
+    every segment in the manifest exists and matched the specifications.
+    However, there is nothing that prevents the user from breaking the SLO
+    download by deleting/replacing a segment referenced in the manifest. It is
+    left to the user to use caution in handling the segments.
+
 
 -----------------------
 Deleting a Large Object
@@ -323,7 +339,6 @@ metadata which can be used for stats and billing purposes.
 """
 
 import base64
-from cgi import parse_header
 from collections import defaultdict
 from datetime import datetime
 import json
@@ -331,8 +346,6 @@ import mimetypes
 import re
 import time
 
-import six
-
 from swift.cli.container_deleter import make_delete_jobs
 from swift.common.header_key_dict import HeaderKeyDict
 from swift.common.exceptions import ListingIterError, SegmentError
@@ -345,21 +358,21 @@ from swift.common.swob import Request, HTTPBadRequest, HTTPServerError, \
     HTTPServiceUnavailable, Response, Range, normalize_etag, \
     RESPONSE_REASONS, str_to_wsgi, bytes_to_wsgi, wsgi_to_str, wsgi_quote
 from swift.common.utils import get_logger, config_true_value, \
-    get_valid_utf8_str, override_bytes_from_content_type, split_path, \
+    override_bytes_from_content_type, split_path, \
     RateLimitedIterator, quote, closing_if_possible, \
     LRUCache, StreamingPile, strict_b64decode, Timestamp, friendly_close, \
-    get_expirer_container, md5
+    md5, parse_header
 from swift.common.registry import register_swift_info
 from swift.common.request_helpers import SegmentedIterable, \
     get_sys_meta_prefix, update_etag_is_at_header, resolve_etag_is_at_header, \
     get_container_update_override_key, update_ignore_range_header, \
-    get_param
-from swift.common.constraints import check_utf8, AUTO_CREATE_ACCOUNT_PREFIX
+    get_param, get_valid_part_num, get_heartbeat_response_body
+from swift.common.constraints import check_utf8
 from swift.common.http import HTTP_NOT_FOUND, HTTP_UNAUTHORIZED
 from swift.common.wsgi import WSGIContext, make_subrequest, make_env, \
     make_pre_authed_request
-from swift.common.middleware.bulk import get_response_body, \
-    ACCEPTABLE_FORMATS, Bulk
+from swift.common.middleware.bulk import ACCEPTABLE_FORMATS, Bulk
+from swift.obj import expirer
 from swift.proxy.controllers.base import get_container_info
 
 
@@ -446,12 +459,12 @@ def parse_and_validate_input(req_body, req_path):
             continue
 
         if segment_type == 'path':
-            if not isinstance(seg_dict['path'], six.string_types):
+            if not isinstance(seg_dict['path'], str):
                 errors.append(b"Index %d: \"path\" must be a string" %
                               seg_index)
                 continue
             if not (seg_dict.get('etag') is None or
-                    isinstance(seg_dict['etag'], six.string_types)):
+                    isinstance(seg_dict['etag'], str)):
                 errors.append(b'Index %d: "etag" must be a string or null '
                               b'(if provided)' % seg_index)
                 continue
@@ -564,6 +577,60 @@ def _annotate_segments(segments, logger=None):
         seg_dict['segment_length'] = segment_length
 
 
+def calculate_byterange_for_part_num(req, segments, part_num):
+    """
+    Helper function to calculate the byterange for a part_num response.
+
+    N.B. as a side-effect of calculating the single tuple representing the
+    byterange required for a part_num response this function will also mutate
+    the request's Range header so that swob knows to return 206.
+
+    :param req: the request object
+    :param segments: the list of seg_dicts
+    :param part_num: the part number of the object to return
+
+    :returns: a tuple representing the byterange
+    """
+    start = 0
+    for seg in segments[:part_num - 1]:
+        start += seg['segment_length']
+    last = start + segments[part_num - 1]['segment_length']
+    # We need to mutate the request's Range header so that swob knows to
+    # handle these partial content requests correctly.
+    req.range = "bytes=%d-%d" % (start, last - 1)
+    return start, last - 1
+
+
+def calculate_byteranges(req, segments, resp_attrs, part_num):
+    """
+    Calculate the byteranges based on the request, segments, and part number.
+
+    N.B. as a side-effect of calculating the single tuple representing the
+    byterange required for a part_num response this function will also mutate
+    the request's Range header so that swob knows to return 206.
+
+    :param req: the request object
+    :param segments: the list of seg_dicts
+    :param resp_attrs: the slo response attributes
+    :param part_num: the part number of the object to return
+
+    :returns: a list of tuples representing byteranges
+    """
+    if req.range:
+        byteranges = [
+            # For some reason, swob.Range.ranges_for_length adds 1 to the
+            # last byte's position.
+            (start, end - 1) for start, end
+            in req.range.ranges_for_length(resp_attrs.slo_size)]
+    elif part_num:
+        byteranges = [
+            calculate_byterange_for_part_num(req, segments, part_num)]
+    else:
+        byteranges = [(0, resp_attrs.slo_size - 1)]
+
+    return byteranges
+
+
 class RespAttrs(object):
     """
     Encapsulate properties of a GET or HEAD response that are pertinent to
@@ -574,7 +641,8 @@ class RespAttrs(object):
 
     :param is_slo: True if the response appears to be an SLO manifest, False
         otherwise.
-    :param timestamp: an instance of :class:`~swift.common.utils.Timestamp`.
+    :param timestamp: a value from which an instance of
+        :class:`~swift.common.utils.Timestamp` can be constructed.
     :param manifest_etag: the Etag of the manifest object, or None if
         ``is_slo`` is False.
     :param slo_etag: the Etag of the SLO.
@@ -582,7 +650,7 @@ class RespAttrs(object):
     """
     def __init__(self, is_slo, timestamp, manifest_etag, slo_etag, slo_size):
         self.is_slo = bool(is_slo)
-        self.timestamp = Timestamp(timestamp or 0)
+        self.timestamp = Timestamp(timestamp or Timestamp.zero())
         # manifest_etag is unambiguous, but json_md5 is even more explicit
         self.json_md5 = manifest_etag or ''
         self.slo_etag = slo_etag or ''
@@ -684,13 +752,14 @@ class SloGetContext(WSGIContext):
             method='GET',
             headers={'x-auth-token': req.headers.get('x-auth-token')},
             agent='%(orig)s SLO MultipartGET', swift_source='SLO')
+        params_copy = dict(req.params)
+        params_copy.pop('part-number', None)
+        sub_req.params = params_copy
         sub_resp = sub_req.get_response(self.slo.app)
 
         if not sub_resp.is_success:
             # Error message should be short
-            body = sub_resp.body
-            if not six.PY2:
-                body = body.decode('utf-8')
+            body = sub_resp.body.decode('utf-8')
             msg = ('while fetching %s, GET of submanifest %s '
                    'failed with status %d (%s)')
             raise ListingIterError(msg % (
@@ -800,10 +869,7 @@ class SloGetContext(WSGIContext):
                         "While processing manifest %r, "
                         "max recursion depth was exceeded" % req.path)
 
-                if six.PY2:
-                    sub_path = get_valid_utf8_str(seg_dict['name'])
-                else:
-                    sub_path = seg_dict['name']
+                sub_path = seg_dict['name']
                 sub_cont, sub_obj = split_path(sub_path, 2, 2, True)
                 if last_sub_path != sub_path:
                     sub_segments = cached_fetch_sub_slo_segments(
@@ -822,8 +888,6 @@ class SloGetContext(WSGIContext):
                         recursion_depth=recursion_depth + 1):
                     yield sub_seg_dict
             else:
-                if six.PY2 and isinstance(seg_dict['name'], six.text_type):
-                    seg_dict['name'] = seg_dict['name'].encode("utf-8")
                 yield dict(seg_dict,
                            first_byte=max(0, first_byte) + range_start,
                            last_byte=min(range_end, range_start + last_byte))
@@ -847,8 +911,7 @@ class SloGetContext(WSGIContext):
         # we can avoid re-fetching the object.
         return first_byte == 0 and last_byte == length - 1
 
-    def _is_manifest_and_need_to_refetch(self, req, resp_attrs,
-                                         is_manifest_get):
+    def _need_to_refetch_manifest(self, req, resp_attrs, is_part_num_request):
         """
         Check if the segments will be needed to service the request and update
         the segment_listing_needed attribute.
@@ -856,19 +919,11 @@ class SloGetContext(WSGIContext):
         :return: boolean indicating if we need to refetch, only if the segments
                  ARE needed we MAY need to refetch them!
         """
-        if not resp_attrs.is_slo:
-            # Not a static large object manifest, maybe an error, regardless
-            # no refetch needed
-            return False
-
-        if is_manifest_get:
-            # Any manifest json object response will do
-            return False
-
         if req.method == 'HEAD':
             # There may be some cases in the future where a HEAD resp on even a
             # modern manifest should refetch, e.g. lp bug #2029174
-            self.segment_listing_needed = resp_attrs.is_legacy
+            self.segment_listing_needed = (resp_attrs.is_legacy or
+                                           is_part_num_request)
             # it will always be the case that a HEAD must re-fetch iff
             # segment_listing_needed
             return self.segment_listing_needed
@@ -916,9 +971,11 @@ class SloGetContext(WSGIContext):
         friendly_close(resp_iter)
         del req.environ['swift.non_client_disconnect']
 
+        headers_subset = ['x-auth-token', 'x-open-expired']
         get_req = make_subrequest(
             req.environ, method='GET',
-            headers={'x-auth-token': req.headers.get('x-auth-token')},
+            headers={k: req.headers.get(k)
+                     for k in headers_subset if k in req.headers},
             agent='%(orig)s SLO MultipartGET', swift_source='SLO')
         resp_iter = self._app_call(get_req.environ)
         new_resp_attrs = RespAttrs.from_headers(self._response_headers)
@@ -965,22 +1022,56 @@ class SloGetContext(WSGIContext):
                                      replace_headers)
 
     def _return_slo_response(self, req, start_response, resp_iter, resp_attrs):
+        headers = {
+            'Etag': '"%s"' % resp_attrs.slo_etag,
+            'X-Manifest-Etag': resp_attrs.json_md5,
+            # swob will fix this for a GET with Range
+            'Content-Length': str(resp_attrs.slo_size),
+            # ignore bogus content-range, make swob figure it out
+            'Content-Range': None,
+        }
         if self.segment_listing_needed:
             # consume existing resp_iter; we'll create a new one
             segments = self._parse_segments(resp_iter)
             resp_attrs.update_from_segments(segments)
-            if req.method == 'HEAD':
+            headers['Etag'] = '"%s"' % resp_attrs.slo_etag
+            headers['Content-Length'] = str(resp_attrs.slo_size)
+            part_num = get_valid_part_num(req)
+            if part_num:
+                headers['X-Parts-Count'] = len(segments)
+
+            if part_num and part_num > len(segments):
+                if req.method == 'HEAD':
+                    resp_iter = []
+                    headers['Content-Length'] = '0'
+                else:
+                    body = b'The requested part number is not satisfiable'
+                    resp_iter = [body]
+                    headers['Content-Length'] = len(body)
+                headers['Content-Range'] = 'bytes */%d' % resp_attrs.slo_size
+                self._response_status = '416 Requested Range Not Satisfiable'
+            elif part_num and req.method == 'HEAD':
+                resp_iter = []
+                headers['Content-Length'] = \
+                    segments[part_num - 1].get('segment_length')
+                start, end = calculate_byterange_for_part_num(
+                    req, segments, part_num)
+                headers['Content-Range'] = \
+                    'bytes {}-{}/{}'.format(start, end,
+                                            resp_attrs.slo_size)
+                # The RFC specifies 206 in the context of Range requests, and
+                # Range headers MUST be ignored for HEADs [1], so a HEAD will
+                # not normally return a 206. However, a part-number HEAD
+                # returns Content-Length equal to the part size, rather than
+                # the whole object size, so in this case we do return 206.
+                # [1] https://www.rfc-editor.org/rfc/rfc9110#name-range
+                self._response_status = '206 Partial Content'
+            elif req.method == 'HEAD':
                 resp_iter = []
             else:
-                resp_iter = self._build_resp_iter(req, segments, resp_attrs)
-        headers = {
-            'Etag': '"%s"' % resp_attrs.slo_etag,
-            'X-Manifest-Etag': resp_attrs.json_md5,
-            # This isn't correct for range requests, but swob will fix it?
-            'Content-Length': str(resp_attrs.slo_size),
-            # ignore bogus content-range, make swob figure it out
-            'Content-Range': None
-        }
+                byteranges = calculate_byteranges(
+                    req, segments, resp_attrs, part_num)
+                resp_iter = self._build_resp_iter(req, segments, byteranges)
         return self._return_response(req, start_response, resp_iter,
                                      replace_headers=headers)
 
@@ -1046,21 +1137,32 @@ class SloGetContext(WSGIContext):
             update_ignore_range_header(req, 'X-Static-Large-Object')
 
         # process original request
+        orig_path_info = req.path_info
         resp_iter = self._app_call(req.environ)
         resp_attrs = RespAttrs.from_headers(self._response_headers)
-        # the next two calls hide a couple side-effects, sorry:
-        #
-        # 1) regardless of the return value the "need_to_refetch" check *may*
-        #    also set self.segment_listing_needed = True (it's commented to
-        #    help you wrap your head around that one, good luck)
-        # 2) if we refetch, we overwrite the current resp_iter and resp_attrs
-        #    variables, partly because we *might* get back a NOT
-        #    resp_attrs.is_slo response (even if we had one to start), but
-        #    hopefully they're just the manifest resp we needed to refetch!
-        if self._is_manifest_and_need_to_refetch(req, resp_attrs,
-                                                 is_manifest_get):
-            resp_attrs, resp_iter = self._refetch_manifest(
-                req, resp_iter, resp_attrs)
+        if resp_attrs.is_slo and not is_manifest_get:
+            try:
+                # only validate part-number if the request is to an SLO
+                part_num = get_valid_part_num(req)
+            except HTTPException:
+                friendly_close(resp_iter)
+                raise
+            # the next two calls hide a couple side effects, sorry:
+            #
+            # 1) regardless of the return value the "need_to_refetch" check
+            #    *may* also set self.segment_listing_needed = True (it's
+            #    commented to help you wrap your head around that one,
+            #    good luck)
+            # 2) if we refetch, we overwrite the current resp_iter and
+            #    resp_attrs variables, partly because we *might* get back a NOT
+            #    resp_attrs.is_slo response (even if we had one to start), but
+            #    hopefully they're just the manifest resp we needed to refetch!
+            if self._need_to_refetch_manifest(req, resp_attrs, part_num):
+                # reset path in case it was modified during original request
+                # (e.g. object versioning might re-write the path)
+                req.path_info = orig_path_info
+                resp_attrs, resp_iter = self._refetch_manifest(
+                    req, resp_iter, resp_attrs)
 
         if not resp_attrs.is_slo:
             # even if the original resp_attrs may have been SLO we may have
@@ -1100,9 +1202,7 @@ class SloGetContext(WSGIContext):
             seg_dict['etag'] = seg_dict.pop('hash', None)
 
         json_data = json.dumps(segments, sort_keys=True)  # convert to string
-        if six.PY3:
-            json_data = json_data.encode('utf-8')
-        return json_data
+        return json_data.encode('utf-8')
 
     def _get_manifest_read(self, resp_iter):
         with closing_if_possible(resp_iter):
@@ -1115,24 +1215,16 @@ class SloGetContext(WSGIContext):
             raise HTTPServerError(msg)
         return segments
 
-    def _build_resp_iter(self, req, segments, resp_attrs):
+    def _build_resp_iter(self, req, segments, byteranges):
         """
         Build a response iterable for a GET request.
 
         :param req: the request object
-        :param resp_attrs: the slo attributes
+        :param segments: the list of seg_dicts
+        :param byteranges: a list of tuples representing byteranges
 
         :returns: a segmented iterable
         """
-        if req.range:
-            byteranges = [
-                # For some reason, swob.Range.ranges_for_length adds 1 to the
-                # last byte's position.
-                (start, end - 1) for start, end
-                in req.range.ranges_for_length(resp_attrs.slo_size)]
-        else:
-            byteranges = [(0, resp_attrs.slo_size - 1)]
-
         ver, account, _junk = req.split_path(3, 3, rest_with_last=True)
         account = wsgi_to_str(account)
         plain_listing_iter = self._segment_listing_iterator(
@@ -1230,11 +1322,7 @@ class StaticLargeObject(object):
             delete_concurrency=delete_concurrency,
             logger=self.logger)
 
-        prefix = AUTO_CREATE_ACCOUNT_PREFIX
-        self.expiring_objects_account = prefix + (
-            conf.get('expiring_objects_account_name') or 'expiring_objects')
-        self.expiring_objects_container_divisor = int(
-            conf.get('expiring_objects_container_divisor', 86400))
+        self.expirer_config = expirer.ExpirerConfig(conf, logger=self.logger)
 
     def handle_multipart_get_or_head(self, req, start_response):
         """
@@ -1299,12 +1387,8 @@ class StaticLargeObject(object):
                 path2indices[seg_dict['path']].append(index)
 
         def do_head(obj_name):
-            if six.PY2:
-                obj_path = '/'.join(['', vrs, account,
-                                     get_valid_utf8_str(obj_name).lstrip('/')])
-            else:
-                obj_path = '/'.join(['', vrs, account,
-                                     str_to_wsgi(obj_name.lstrip('/'))])
+            obj_path = '/'.join(['', vrs, account,
+                                 str_to_wsgi(obj_name.lstrip('/'))])
             obj_path = wsgi_quote(obj_path)
 
             sub_req = make_subrequest(
@@ -1443,7 +1527,7 @@ class StaticLargeObject(object):
                     start_response(err.status,
                                    [(h, v) for h, v in err.headers.items()
                                     if h.lower() != 'content-length'])
-                yield separator + get_response_body(
+                yield separator + get_heartbeat_response_body(
                     out_content_type, resp_dict, problem_segments, 'upload')
                 return
 
@@ -1456,7 +1540,7 @@ class StaticLargeObject(object):
                     r = '%s:%s;' % (seg_data['hash'], seg_data['range'])
                 else:
                     r = seg_data['hash']
-                slo_etag.update(r.encode('ascii') if six.PY3 else r)
+                slo_etag.update(r.encode('ascii'))
 
             slo_etag = slo_etag.hexdigest()
             client_etag = normalize_etag(req.headers.get('Etag'))
@@ -1466,11 +1550,11 @@ class StaticLargeObject(object):
                     resp_dict = {}
                     resp_dict['Response Status'] = err.status
                     err_body = err.body
-                    if six.PY3 and isinstance(err_body, bytes):
+                    if isinstance(err_body, bytes):
                         err_body = err_body.decode('utf-8', errors='replace')
                     resp_dict['Response Body'] = err_body or '\n'.join(
                         RESPONSE_REASONS.get(err.status_int, ['']))
-                    yield separator + get_response_body(
+                    yield separator + get_heartbeat_response_body(
                         out_content_type, resp_dict, problem_segments,
                         'upload')
                 else:
@@ -1478,9 +1562,7 @@ class StaticLargeObject(object):
                         yield chunk
                 return
 
-            json_data = json.dumps(data_for_storage)
-            if six.PY3:
-                json_data = json_data.encode('utf-8')
+            json_data = json.dumps(data_for_storage).encode('utf-8')
             req.body = json_data
             req.headers.update({
                 SYSMETA_SLO_ETAG: slo_etag,
@@ -1515,10 +1597,10 @@ class StaticLargeObject(object):
 
             if heartbeat:
                 resp_body = resp.body
-                if six.PY3 and isinstance(resp_body, bytes):
+                if isinstance(resp_body, bytes):
                     resp_body = resp_body.decode('utf-8')
                 resp_dict['Response Body'] = resp_body
-                yield separator + get_response_body(
+                yield separator + get_heartbeat_response_body(
                     out_content_type, resp_dict, [], 'upload')
             else:
                 for chunk in resp(req.environ, start_response):
@@ -1541,10 +1623,7 @@ class StaticLargeObject(object):
             raise HTTPPreconditionFailed(
                 request=req, body='Invalid UTF8 or contains NULL')
         vrs, account, container, obj = req.split_path(4, 4, True)
-        if six.PY2:
-            obj_path = ('/%s/%s' % (container, obj)).decode('utf-8')
-        else:
-            obj_path = '/%s/%s' % (wsgi_to_str(container), wsgi_to_str(obj))
+        obj_path = '/%s/%s' % (wsgi_to_str(container), wsgi_to_str(obj))
 
         segments = [{
             'sub_slo': True,
@@ -1572,7 +1651,7 @@ class StaticLargeObject(object):
                 except HTTPException as err:
                     # allow bulk delete response to report errors
                     err_body = err.body
-                    if six.PY3 and isinstance(err_body, bytes):
+                    if isinstance(err_body, bytes):
                         err_body = err_body.decode('utf-8', errors='replace')
                     seg_data['error'] = {'code': err.status_int,
                                          'message': err_body}
@@ -1581,8 +1660,6 @@ class StaticLargeObject(object):
                 seg_data['sub_slo'] = False
                 segments.append(seg_data)
             else:
-                if six.PY2:
-                    seg_data['name'] = seg_data['name'].encode('utf-8')
                 yield seg_data
 
     def get_slo_segments(self, obj_name, req):
@@ -1611,15 +1688,9 @@ class StaticLargeObject(object):
         new_env['HTTP_USER_AGENT'] = \
             '%s MultipartDELETE' % new_env.get('HTTP_USER_AGENT')
         new_env['swift.source'] = 'SLO'
-        if six.PY2:
-            new_env['PATH_INFO'] = (
-                '/%s/%s/%s' % (vrs, account,
-                               obj_name.lstrip('/').encode('utf-8'))
-            )
-        else:
-            new_env['PATH_INFO'] = (
-                '/%s/%s/%s' % (vrs, account, str_to_wsgi(obj_name.lstrip('/')))
-            )
+        new_env['PATH_INFO'] = (
+            '/%s/%s/%s' % (vrs, account, str_to_wsgi(obj_name.lstrip('/')))
+        )
         # Just request the last byte of non-SLO objects so we don't waste
         # a resources in friendly_close() below
         manifest_req = Request.blank('', new_env, range='bytes=-1')
@@ -1654,10 +1725,7 @@ class StaticLargeObject(object):
             raise HTTPPreconditionFailed(
                 request=req, body='Invalid UTF8 or contains NULL')
         vrs, account, container, obj = req.split_path(4, 4, True)
-        if six.PY2:
-            obj_path = ('/%s/%s' % (container, obj)).decode('utf-8')
-        else:
-            obj_path = '/%s/%s' % (wsgi_to_str(container), wsgi_to_str(obj))
+        obj_path = '/%s/%s' % (wsgi_to_str(container), wsgi_to_str(obj))
         segments = [seg for seg in self.get_slo_segments(obj_path, req)
                     if 'data' not in seg]
         if not segments:
@@ -1702,13 +1770,14 @@ class StaticLargeObject(object):
         ts = req.ensure_x_timestamp()
         expirer_jobs = make_delete_jobs(
             wsgi_to_str(account), segment_container, segment_objects, ts)
-        expirer_cont = get_expirer_container(
-            ts, self.expiring_objects_container_divisor,
-            wsgi_to_str(account), wsgi_to_str(container), wsgi_to_str(obj))
+        expiring_objects_account, expirer_cont = \
+            self.expirer_config.get_expirer_account_and_container(
+                ts, wsgi_to_str(account), wsgi_to_str(container),
+                wsgi_to_str(obj))
         enqueue_req = make_pre_authed_request(
             req.environ,
             method='UPDATE',
-            path="/v1/%s/%s" % (self.expiring_objects_account, expirer_cont),
+            path="/v1/%s/%s" % (expiring_objects_account, expirer_cont),
             body=json.dumps(expirer_jobs),
             headers={'Content-Type': 'application/json',
                      'X-Backend-Storage-Policy-Index': '0',
