@@ -142,14 +142,10 @@ To enable this new mode in a Swift cluster the ``versioned_writes`` and
 the option ``allow_object_versioning`` to ``True``.
 """
 
-import calendar
 import itertools
 import json
-import six
-import time
 
-from cgi import parse_header
-from six.moves.urllib.parse import unquote
+from urllib.parse import unquote
 
 from swift.common.constraints import MAX_FILE_SIZE, valid_api_version, \
     ACCOUNT_LISTING_LIMIT, CONTAINER_LISTING_LIMIT
@@ -166,11 +162,11 @@ from swift.common.swob import HTTPPreconditionFailed, HTTPServiceUnavailable, \
     HTTPBadRequest, str_to_wsgi, bytes_to_wsgi, wsgi_quote, \
     wsgi_to_str, wsgi_unquote, Request, HTTPNotFound, HTTPException, \
     HTTPRequestEntityTooLarge, HTTPInternalServerError, HTTPNotAcceptable, \
-    HTTPConflict
+    HTTPConflict, HTTPLengthRequired, parse_date_header
 from swift.common.storage_policy import POLICIES
 from swift.common.utils import get_logger, Timestamp, drain_and_close, \
     config_true_value, close_if_possible, closing_if_possible, \
-    FileLikeIter, split_path, parse_content_type, RESERVED_STR
+    FileLikeIter, split_path, parse_content_type, parse_header, RESERVED_STR
 from swift.common.wsgi import WSGIContext, make_pre_authed_request
 from swift.proxy.controllers.base import get_container_info
 
@@ -236,6 +232,10 @@ class ObjectVersioningContext(WSGIContext):
         return get_reserved_name('versions', container_name)
 
     def _build_versions_object_name(self, object_name, ts):
+        """
+        :param object_name: (str) name of object
+        :param ts: (str) timestamp of object version
+        """
         inv = ~Timestamp(ts)
         return get_reserved_name(object_name, inv.internal)
 
@@ -449,9 +449,7 @@ class ObjectContext(ObjectVersioningContext):
         # X-Versions-Location
         ts_source = get_resp.headers.get(
             'x-timestamp',
-            calendar.timegm(time.strptime(
-                get_resp.headers['last-modified'],
-                '%a, %d %b %Y %H:%M:%S GMT')))
+            str(parse_date_header(get_resp.headers['last-modified'])))
         vers_obj_name = self._build_versions_object_name(
             object_name, ts_source)
 
@@ -673,8 +671,10 @@ class ObjectContext(ObjectVersioningContext):
         Handle a PUT?version-id request and create/update the is_latest link to
         point to the specific version. Expects a valid 'version' id.
         """
-        if req.content_length is None:
+        if req.is_chunked:
             has_body = (req.body_file.read(1) != b'')
+        elif req.content_length is None:
+            raise HTTPLengthRequired(request=req)
         else:
             has_body = (req.content_length != 0)
         if has_body:
@@ -1126,7 +1126,7 @@ class ContainerContext(ObjectVersioningContext):
                     raise HTTPBadRequest('invalid version_marker param')
 
                 params['marker'] = self._build_versions_object_name(
-                    params['marker'], ts)
+                    params['marker'], ts.internal)
         elif 'marker' in params:
             params['marker'] = self._build_versions_object_prefix(
                 params['marker']) + ':'  # just past all numbers
@@ -1190,7 +1190,6 @@ class ContainerContext(ObjectVersioningContext):
                 name, ts = self._split_version_from_name(linked_name)
                 if ts is None:
                     continue
-                name = name.decode('utf8') if six.PY2 else name
                 is_latest = False
                 if name not in is_latest_set:
                     is_latest_set.add(name)
@@ -1231,8 +1230,7 @@ class ContainerContext(ObjectVersioningContext):
                         path = '/v1/%s/%s/%s' % (
                             wsgi_to_str(account),
                             wsgi_to_str(location),
-                            item['name'].encode('utf8')
-                            if six.PY2 else item['name'])
+                            item['name'])
 
                         if path in current_versions:
                             item['is_latest'] = True
@@ -1260,7 +1258,7 @@ class ContainerContext(ObjectVersioningContext):
                     if ts is None:
                         continue
                     broken_listing.append({
-                        'name': name.decode('utf8') if six.PY2 else name,
+                        'name': name,
                         'is_latest': True,
                         'version_id': ts.internal,
                         'content_type': item['content_type'],
