@@ -21,10 +21,10 @@ from swob in here without creating circular imports.
 """
 
 import itertools
-import sys
 import time
+import json
+from xml.sax.saxutils import escape  # nosec B406
 
-import six
 from swift.common.header_key_dict import HeaderKeyDict
 
 from swift.common.constraints import AUTO_CREATE_ACCOUNT_PREFIX, \
@@ -51,15 +51,6 @@ USE_REPLICATION_NETWORK_HEADER = 'x-backend-use-replication-network'
 MISPLACED_OBJECTS_ACCOUNT = '.misplaced_objects'
 
 
-if six.PY2:
-    import cgi
-
-    def html_escape(s, quote=True):
-        return cgi.escape(s, quote=quote)
-else:
-    from html import escape as html_escape  # noqa: F401
-
-
 def get_param(req, name, default=None):
     """
     Get a parameter from an HTTP request ensuring proper handling UTF-8
@@ -68,33 +59,56 @@ def get_param(req, name, default=None):
     :param req: request object
     :param name: parameter name
     :param default: result to return if the parameter is not found
-    :returns: HTTP request parameter value, as a native string
-              (in py2, as UTF-8 encoded str, not unicode object)
+    :returns: HTTP request parameter value, as a native (not WSGI) string
     :raises HTTPBadRequest: if param not valid UTF-8 byte sequence
     """
     value = req.params.get(name, default)
-    if six.PY2:
-        if value and not isinstance(value, six.text_type):
-            try:
-                value.decode('utf8')    # Ensure UTF8ness
-            except UnicodeDecodeError:
-                raise HTTPBadRequest(
-                    request=req, content_type='text/plain',
-                    body='"%s" parameter not valid UTF-8' % name)
-    else:
-        if value:
-            # req.params is a dict of WSGI strings, so encoding will succeed
-            value = value.encode('latin1')
-            try:
-                # Ensure UTF8ness since we're at it
-                value = value.decode('utf8')
-            except UnicodeDecodeError:
-                raise HTTPBadRequest(
-                    request=req, content_type='text/plain',
-                    body='"%s" parameter not valid UTF-8' % name)
+    if value:
+        # req.params is a dict of WSGI strings, so encoding will succeed
+        value = value.encode('latin1')
+        try:
+            # Ensure UTF8ness since we're at it
+            value = value.decode('utf8')
+        except UnicodeDecodeError:
+            raise HTTPBadRequest(
+                request=req, content_type='text/plain',
+                body='"%s" parameter not valid UTF-8' % name)
     return value
 
 
+def get_valid_part_num(req):
+    """
+    Any non-range GET or HEAD request for a SLO object may include a
+    part-number parameter in query string.  If the passed in request
+    includes a part-number parameter it will be parsed into a valid integer
+    and returned.  If the passed in request does not include a part-number
+    param we will return None.  If the part-number parameter is invalid for
+    the given request we will raise the appropriate HTTP exception
+
+    :param req: the request object
+
+    :returns: validated part-number value or None
+    :raises HTTPBadRequest: if request or part-number param is not valid
+    """
+    part_number_param = get_param(req, 'part-number')
+    if part_number_param is None:
+        return None
+    try:
+        part_number = int(part_number_param)
+        if part_number <= 0:
+            raise ValueError
+    except ValueError:
+        raise HTTPBadRequest('Part number must be an integer greater '
+                             'than 0')
+
+    if req.range:
+        raise HTTPBadRequest(req=req,
+                             body='Range requests are not supported '
+                                  'with part number queries')
+
+    return part_number
+
+
 def validate_params(req, names):
     """
     Get list of parameters from an HTTP request, validating the encoding of
@@ -530,8 +544,8 @@ class SegmentedIterable(object):
                 path = quote(seg_path) + '?multipart-manifest=get'
                 seg_req = make_subrequest(
                     self.req.environ, path=path, method='GET',
-                    headers={'x-auth-token': self.req.headers.get(
-                        'x-auth-token')},
+                    headers={h: self.req.headers.get(h)
+                             for h in ('x-auth-token', 'x-open-expired')},
                     agent=('%(orig)s ' + self.ua_suffix),
                     swift_source=self.swift_source)
 
@@ -573,11 +587,10 @@ class SegmentedIterable(object):
                 pending_etag = seg_etag
                 pending_size = seg_size
 
-        except ListingIterError:
-            e_type, e_value, e_traceback = sys.exc_info()
+        except ListingIterError as e:
             if pending_req:
                 yield pending_req, pending_etag, pending_size
-            six.reraise(e_type, e_value, e_traceback)
+            raise e
 
         if pending_req:
             yield pending_req, pending_etag, pending_size
@@ -596,9 +609,7 @@ class SegmentedIterable(object):
             seg_resp = seg_req.get_response(self.app)
             if not is_success(seg_resp.status_int):
                 # Error body should be short
-                body = seg_resp.body
-                if not six.PY2:
-                    body = body.decode('utf8')
+                body = seg_resp.body.decode('utf8')
                 msg = 'While processing manifest %s, got %d (%s) ' \
                     'while retrieving %s' % (
                         self.name, seg_resp.status_int,
@@ -960,3 +971,83 @@ def get_ip_port(node, headers):
     """
     return select_ip_port(
         node, use_replication=is_use_replication_network(headers))
+
+
+def is_open_expired(app, req):
+    """
+    Helper function to check if a request with the header 'x-open-expired'
+    can access an object that has not yet been reaped by the object-expirer
+    based on the allow_open_expired global config.
+
+    :param app: the application instance
+    :param req: request object
+    """
+    return (config_true_value(app.allow_open_expired) and
+            config_true_value(req.headers.get('x-open-expired')))
+
+
+def is_backend_open_expired(request):
+    """
+    Helper function to check if a request has either the headers
+    'x-backend-open-expired' or 'x-backend-replication' for the backend
+    to access expired objects.
+
+    :param request: request object
+    """
+    x_backend_open_expired = config_true_value(request.headers.get(
+        'x-backend-open-expired', 'false'))
+    x_backend_replication = config_true_value(request.headers.get(
+        'x-backend-replication', 'false'))
+    return x_backend_open_expired or x_backend_replication
+
+
+def append_log_info(environ, log_info):
+    environ.setdefault('swift.log_info', []).append(log_info)
+
+
+def get_log_info(environ):
+    return ','.join(environ.get('swift.log_info', []))
+
+
+def get_heartbeat_response_body(data_format, data_dict, error_list, root_tag):
+    """
+    Returns a response body for heartbeat according to format.
+
+    Handles json and xml, otherwise will return text/plain.
+    Note: xml response does not include xml declaration.
+
+    :params data_format: resulting format
+    :params data_dict: generated data about results.
+    :params error_list: list of quoted filenames that failed
+    :params root_tag: the tag name to use for root elements when returning XML;
+                      e.g. 'extract' or 'delete'
+    """
+    if data_format == 'application/json':
+        data_dict['Errors'] = error_list
+        return json.dumps(data_dict).encode('ascii')
+    if data_format and data_format.endswith('/xml'):
+        output = ['<', root_tag, '>\n']
+        for key in sorted(data_dict):
+            xml_key = key.replace(' ', '_').lower()
+            output.extend([
+                '<', xml_key, '>',
+                escape(str(data_dict[key])),
+                '</', xml_key, '>\n',
+            ])
+        output.append('<errors>\n')
+        for name, status in error_list:
+            output.extend([
+                '<object><name>', escape(name), '</name><status>',
+                escape(status), '</status></object>\n',
+            ])
+        output.extend(['</errors>\n</', root_tag, '>\n'])
+        return ''.join(output).encode('utf-8')
+
+    output = []
+    for key in sorted(data_dict):
+        output.append('%s: %s\n' % (key, data_dict[key]))
+    output.append('Errors:\n')
+    output.extend(
+        '%s, %s\n' % (name, status)
+        for name, status in error_list)
+    return ''.join(output).encode('utf-8')
