@@ -15,12 +15,12 @@
 
 """WSGI tools for use with swift."""
 
-from __future__ import print_function
 
 import errno
-import fcntl
+import json
 import os
 import signal
+import struct
 import sys
 from textwrap import dedent
 import time
@@ -31,10 +31,7 @@ import eventlet.debug
 from eventlet import greenio, GreenPool, sleep, wsgi, listen, Timeout
 from paste.deploy import loadwsgi
 from eventlet.green import socket, ssl, os as green_os
-from io import BytesIO
-
-import six
-from six import StringIO
+from io import BytesIO, StringIO
 
 from swift.common import utils, constraints
 from swift.common.http_protocol import SwiftHttpProtocol, \
@@ -44,11 +41,13 @@ from swift.common.swob import Request, wsgi_unquote
 from swift.common.utils import capture_stdio, disable_fallocate, \
     drop_privileges, get_logger, NullLogger, config_true_value, \
     validate_configuration, get_hub, config_auto_int_value, \
-    reiterate, clean_up_daemon_hygiene, systemd_notify, NicerInterpolation
+    reiterate, clean_up_daemon_hygiene, systemd_notify, NicerInterpolation, \
+    set_swift_dir
 
 SIGNUM_TO_NAME = {getattr(signal, n): n for n in dir(signal)
                   if n.startswith('SIG') and '_' not in n}
 NOTIFY_FD_ENV_KEY = '__SWIFT_SERVER_NOTIFY_FD'
+CHILD_STATE_FD_ENV_KEY = '__SWIFT_SERVER_CHILD_STATE_FD'
 
 # Set maximum line size of message headers to be accepted.
 wsgi.MAX_HEADER_LINE = constraints.MAX_HEADER_SIZE
@@ -67,8 +66,7 @@ class NamedConfigLoader(loadwsgi.ConfigLoader):
     """
 
     def get_context(self, object_type, name=None, global_conf=None):
-        if not six.PY2:
-            self.parser._interpolation = NicerInterpolation()
+        self.parser._interpolation = NicerInterpolation()
         context = super(NamedConfigLoader, self).get_context(
             object_type, name=name, global_conf=global_conf)
         context.name = name
@@ -128,10 +126,7 @@ class ConfigString(NamedConfigLoader):
         self.parser.optionxform = str  # Don't lower-case keys
         # Defaults don't need interpolation (crazy PasteDeploy...)
         self.parser.defaults = lambda: dict(self.parser._defaults, **defaults)
-        if six.PY2:
-            self.parser.readfp(self.contents)
-        else:
-            self.parser.read_file(self.contents)
+        self.parser.read_file(self.contents)
 
     def readline(self, *args, **kwargs):
         return self.contents.readline(*args, **kwargs)
@@ -198,10 +193,7 @@ def get_socket(conf):
             sock = listen(bind_addr, backlog=int(conf.get('backlog', 4096)),
                           family=address_family)
             if 'cert_file' in conf:
-                if six.PY2:
-                    context = ssl.SSLContext(ssl.PROTOCOL_SSLv23)
-                else:
-                    context = ssl.SSLContext(ssl.PROTOCOL_TLS_SERVER)
+                context = ssl.SSLContext(ssl.PROTOCOL_TLS_SERVER)
                 context.verify_mode = ssl.CERT_NONE
                 context.load_cert_chain(conf['cert_file'], conf['key_file'])
                 warn_ssl = True
@@ -443,6 +435,7 @@ def run_server(conf, logger, sock, global_conf=None, ready_callback=None,
     else:
         protocol_class = SwiftHttpProtocol
 
+    max_request_line = constraints.MAX_REQUEST_LINE
     server_kwargs = {
         'custom_pool': pool,
         'protocol': protocol_class,
@@ -451,6 +444,7 @@ def run_server(conf, logger, sock, global_conf=None, ready_callback=None,
         # the AWS SDK to work with s3api middleware (it needs an "ETag"
         # header; "Etag" just won't do).
         'capitalize_response_headers': False,
+        'url_length_limit': max_request_line,
     }
     if conf.get('keepalive_timeout'):
         server_kwargs['keepalive'] = float(conf['keepalive_timeout']) or False
@@ -485,6 +479,13 @@ class StrategyBase(object):
         # children to easily drop refs to sibling sockets in post_fork_hook().
         self.tracking_data = {}
 
+        # When doing a seamless reload, we inherit a bunch of child processes
+        # that should all clean themselves up fairly quickly; track them here
+        self.reload_pids = dict()
+        # If they don't cleanup quickly, we'll start killing them after this
+        self.stale_worker_timeout = utils.non_negative_float(
+            conf.get('stale_worker_timeout', 86400))
+
     def post_fork_hook(self):
         """
         Called in each forked-off child process, prior to starting the actual
@@ -513,13 +514,10 @@ class StrategyBase(object):
         """
 
         for sock in self.iter_sockets():
-            if six.PY2:
-                fcntl.fcntl(sock.fileno(), fcntl.F_SETFD, fcntl.FD_CLOEXEC)
-            else:
-                # Python 3.4 and later default to sockets having close-on-exec
-                # set (what PEP 0446 calls "non-inheritable").  This new method
-                # on socket objects is provided to toggle it.
-                sock.set_inheritable(False)
+            # Python 3.4 and later default to sockets having close-on-exec
+            # set (what PEP 0446 calls "non-inheritable").  This new method
+            # on socket objects is provided to toggle it.
+            sock.set_inheritable(False)
 
     def signal_ready(self):
         """
@@ -537,9 +535,21 @@ class StrategyBase(object):
         # connections. This is used for seamless reloading using SIGUSR1.
         reexec_signal_fd = os.getenv(NOTIFY_FD_ENV_KEY)
         if reexec_signal_fd:
+            if ',' in reexec_signal_fd:
+                reexec_signal_fd, worker_state_fd = reexec_signal_fd.split(',')
             reexec_signal_fd = int(reexec_signal_fd)
             os.write(reexec_signal_fd, str(os.getpid()).encode('utf8'))
             os.close(reexec_signal_fd)
+        worker_state_fd = os.getenv(CHILD_STATE_FD_ENV_KEY)
+        try:
+            self.read_state_from_old_manager(worker_state_fd)
+        except Exception as e:
+            # This was all opportunistic anyway; old swift wouldn't even
+            # *try* to send us any state -- we don't want *new* code to
+            # fail just because *old* code didn't live up to its promise
+            self.logger.warning(
+                'Failed to read state from the old manager: %r', e,
+                exc_info=True)
 
         # Finally, signal systemd (if appropriate) that process started
         # properly.
@@ -547,6 +557,110 @@ class StrategyBase(object):
 
         self.signaled_ready = True
 
+    def read_state_from_old_manager(self, worker_state_fd):
+        """
+        Read worker state from the old manager's socket-closer.
+
+        The socket-closing process is the last thing to still have the worker
+        PIDs in its head, so it sends us a JSON dict (prefixed by its length)
+        of the form::
+
+           {
+             "old_pids": {
+               "<old worker>": "<first reload time>",
+               ...
+             }
+           }
+
+        More data may be added in the future.
+
+        :param worker_state_fd: The file descriptor that should have the
+                                old worker state. Should be passed to us
+                                via the ``__SWIFT_SERVER_CHILD_STATE_FD``
+                                environment variable.
+        """
+        if not worker_state_fd:
+            return
+        worker_state_fd = int(worker_state_fd)
+        try:
+            # The temporary manager may have up and died while trying to send
+            # state; hopefully its logs will have more about what went wrong
+            # -- let's just log at warning here
+            data_len = os.read(worker_state_fd, 4)
+            if len(data_len) != 4:
+                self.logger.warning(
+                    'Invalid worker state received; expected 4 bytes '
+                    'followed by a payload but only received %d bytes',
+                    len(data_len))
+                return
+
+            data_len = struct.unpack('!I', data_len)[0]
+            data = b''
+            while len(data) < data_len:
+                chunk = os.read(worker_state_fd, data_len - len(data))
+                if not chunk:
+                    break
+                data += chunk
+            if len(data) != data_len:
+                self.logger.warning(
+                    'Incomplete worker state received; expected %d '
+                    'bytes but only received %d', data_len, len(data))
+                return
+
+            # OK, the temporary manager was able to tell us how much it wanted
+            # to send and send it; from here on, error seems appropriate.
+            try:
+                old_state = json.loads(data)
+            except ValueError:
+                self.logger.error(
+                    'Invalid worker state received; '
+                    'invalid JSON: %r', data)
+                return
+
+            try:
+                old_pids = {
+                    int(pid): float(reloaded)
+                    for pid, reloaded in old_state["old_pids"].items()}
+            except (KeyError, TypeError) as err:
+                self.logger.error(
+                    'Invalid worker state received; '
+                    'error reading old pids: %s', err)
+            self.logger.debug('Received old worker pids: %s', old_pids)
+            self.reload_pids.update(old_pids)
+
+            def smother(old_pids=old_pids, timeout=self.stale_worker_timeout):
+                own_pid = os.getpid()
+                kill_times = sorted(((reloaded + timeout, pid)
+                                     for pid, reloaded in old_pids.items()),
+                                    reverse=True)
+                while kill_times:
+                    kill_time, pid = kill_times.pop()
+                    now = time.time()
+                    if kill_time > now:
+                        sleep(kill_time - now)
+                    try:
+                        ppid = utils.get_ppid(pid)
+                    except OSError as e:
+                        if e.errno != errno.ESRCH:
+                            self.logger.error("Could not determine parent "
+                                              "for stale pid %d: %s", pid, e)
+                        continue
+                    if ppid == own_pid:
+                        self.logger.notice("Killing long-running stale worker "
+                                           "%d after %ds", pid, int(timeout))
+                        try:
+                            os.kill(pid, signal.SIGKILL)
+                        except OSError as e:
+                            if e.errno != errno.ESRCH:
+                                self.logger.error(
+                                    "Could not kill stale pid %d: %s", pid, e)
+                    # else, pid got re-used?
+
+            eventlet.spawn_n(smother)
+
+        finally:
+            os.close(worker_state_fd)
+
 
 class WorkersStrategy(StrategyBase):
     """
@@ -636,11 +750,17 @@ class WorkersStrategy(StrategyBase):
         :param int pid: The PID of the worker that exited.
         """
 
+        if self.reload_pids.pop(pid, None):
+            self.logger.notice('Removing stale child %d from parent %d',
+                               pid, os.getpid())
+            return
+
         sock = self.tracking_data.pop(pid, None)
         if sock is None:
-            self.logger.info('Ignoring wait() result from unknown PID %s', pid)
+            self.logger.warning('Ignoring wait() result from unknown PID %d',
+                                pid)
         else:
-            self.logger.error('Removing dead child %s from parent %s',
+            self.logger.error('Removing dead child %d from parent %d',
                               pid, os.getpid())
             greenio.shutdown_safe(sock)
             sock.close()
@@ -653,6 +773,9 @@ class WorkersStrategy(StrategyBase):
         for sock in self.tracking_data.values():
             yield sock
 
+    def get_worker_pids(self):
+        return list(self.tracking_data.keys())
+
 
 class ServersPerPortStrategy(StrategyBase):
     """
@@ -800,14 +923,23 @@ class ServersPerPortStrategy(StrategyBase):
         :param int pid: The PID of the worker that exited.
         """
 
+        if self.reload_pids.pop(pid, None):
+            self.logger.notice('Removing stale child %d from parent %d',
+                               pid, os.getpid())
+            return
+
         for port_data in self.tracking_data.values():
             for idx, (child_pid, sock) in enumerate(port_data):
                 if child_pid == pid:
+                    self.logger.error('Removing dead child %d from parent %d',
+                                      pid, os.getpid())
                     port_data[idx] = (None, None)
                     greenio.shutdown_safe(sock)
                     sock.close()
                     return
 
+        self.logger.warning('Ignoring wait() result from unknown PID %d', pid)
+
     def iter_sockets(self):
         """
         Yields all known listen sockets.
@@ -817,6 +949,12 @@ class ServersPerPortStrategy(StrategyBase):
             for _pid, sock in port_data:
                 yield sock
 
+    def get_worker_pids(self):
+        return [
+            pid
+            for port_data in self.tracking_data.values()
+            for pid, _sock in port_data]
+
 
 def check_config(conf_path, app_section, *args, **kwargs):
     # Load configuration, Set logger and Load request processor
@@ -1014,24 +1152,29 @@ def run_wsgi(conf_path, app_section, *args, **kwargs):
         # then the old server can't actually ever exit.
         strategy.set_close_on_exec_on_listen_sockets()
         read_fd, write_fd = os.pipe()
+        state_rfd, state_wfd = os.pipe()
         orig_server_pid = os.getpid()
         child_pid = os.fork()
         if child_pid:
             # parent; set env var for fds and reexec ourselves
             os.close(read_fd)
+            os.close(state_wfd)
             os.putenv(NOTIFY_FD_ENV_KEY, str(write_fd))
+            os.putenv(CHILD_STATE_FD_ENV_KEY, str(state_rfd))
             myself = os.path.realpath(sys.argv[0])
             logger.info("Old server PID=%d re'execing as: %r",
                         orig_server_pid, [myself] + list(sys.argv))
             if hasattr(os, 'set_inheritable'):
                 # See https://www.python.org/dev/peps/pep-0446/
                 os.set_inheritable(write_fd, True)
-            os.execv(myself, sys.argv)
+                os.set_inheritable(state_rfd, True)
+            os.execv(myself, sys.argv)  # nosec B606
             logger.error('Somehow lived past os.execv()?!')
             exit('Somehow lived past os.execv()?!')
         elif child_pid == 0:
             # child
             os.close(write_fd)
+            os.close(state_rfd)
             logger.info('Old server temporary child PID=%d waiting for '
                         "re-exec'ed PID=%d to signal readiness...",
                         os.getpid(), orig_server_pid)
@@ -1046,6 +1189,16 @@ def run_wsgi(conf_path, app_section, *args, **kwargs):
                     logger.info('Old server temporary child PID=%d notified '
                                 'to shutdown old listen sockets by PID=%s',
                                 os.getpid(), got_pid)
+                    # Ensure new process knows about old children
+                    stale_pids = dict(strategy.reload_pids)
+                    stale_pids[os.getpid()] = now = time.time()
+                    stale_pids.update({
+                        pid: now for pid in strategy.get_worker_pids()})
+                    data = json.dumps({
+                        "old_pids": stale_pids,
+                    }).encode('ascii')
+                    os.write(state_wfd, struct.pack('!I', len(data)) + data)
+                    os.close(state_wfd)
                 else:
                     logger.warning('Old server temporary child PID=%d *NOT* '
                                    'notified to shutdown old listen sockets; '
@@ -1079,6 +1232,9 @@ def _initrp(conf_path, app_section, *args, **kwargs):
         raise ConfigFileError("Error trying to load config from %s: %s" %
                               (conf_path, e))
 
+    if conf.get('swift_dir'):
+        set_swift_dir(conf['swift_dir'])
+
     validate_configuration()
 
     # pre-configure logger
@@ -1202,7 +1358,7 @@ def make_env(env, method=None, path=None, agent='Swift', query_string=None,
                  'swift.trans_id', 'swift.authorize_override',
                  'swift.authorize', 'HTTP_X_USER_ID', 'HTTP_X_PROJECT_ID',
                  'HTTP_REFERER', 'swift.infocache',
-                 'swift.shard_listing_history'):
+                 'swift.base_labels', 'swift.shard_listing_history'):
         if name in env:
             newenv[name] = env[name]
     if method:
