@@ -12,9 +12,9 @@
 # implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from six.moves import queue
+import queue
 
-import six.moves.cPickle as pickle
+import pickle  # nosec: B403
 import errno
 import os
 import signal
@@ -22,6 +22,7 @@ import sys
 import time
 import uuid
 from random import random, shuffle
+from bisect import insort
 from collections import deque
 
 from eventlet import spawn, Timeout
@@ -34,7 +35,8 @@ from swift.common.utils import get_logger, renamer, write_pickle, \
     dump_recon_cache, config_true_value, RateLimitedIterator, split_path, \
     eventlet_monkey_patch, get_redirect_data, ContextPool, hash_path, \
     non_negative_float, config_positive_int_value, non_negative_int, \
-    EventletRateLimiter, node_to_string, parse_options
+    EventletRateLimiter, node_to_string, parse_options, load_recon_cache, \
+    Timestamp
 from swift.common.daemon import Daemon, run_daemon
 from swift.common.header_key_dict import HeaderKeyDict
 from swift.common.storage_policy import split_policy_string, PolicyError
@@ -61,8 +63,6 @@ class RateLimiterBucket(EventletRateLimiter):
     def __bool__(self):
         return bool(self.deque)
 
-    __nonzero__ = __bool__  # py2
-
     def __lt__(self, other):
         # used to sort RateLimiterBuckets by readiness
         if isinstance(other, RateLimiterBucket):
@@ -144,7 +144,7 @@ class BucketizedUpdateSkippingLimiter(object):
     def _get_time(self):
         return time.time()
 
-    def next(self):
+    def __next__(self):
         # first iterate over the wrapped iterator...
         for update_ctx in self.iterator:
             bucket = self.buckets[self._bucket_key(update_ctx['update'])]
@@ -212,7 +212,116 @@ class BucketizedUpdateSkippingLimiter(object):
 
         raise StopIteration()
 
-    __next__ = next
+
+class OldestAsyncPendingTracker:
+    """
+    Manages the tracking of the oldest async pending updates for each
+    account-container pair using a sorted list for timestamps. Evicts the
+    newest pairs when t max_entries is reached. Supports retrieving the N
+    oldest async pending updates or calculating the age of the oldest pending
+    update.
+    """
+    def __init__(
+        self,
+        max_entries,
+    ):
+        self.max_entries = max_entries
+        self.sorted_entries = []
+        self.ac_to_timestamp = {}
+
+    def add_update(self, account, container, timestamp):
+        """
+        Add or update a timestamp for a given account and container.
+
+        :param account: (str) The account name.
+        :param container: (str) The container name.
+        :param timestamp: (str) The timestamp to add or update.
+        """
+        # Ensure the timestamp is a float
+        timestamp = float(Timestamp(timestamp))
+
+        ac = (account, container)
+
+        if ac in self.ac_to_timestamp:
+            old_timestamp = self.ac_to_timestamp[ac]
+            # Only replace the existing timestamp if the new one is older
+            if timestamp < old_timestamp:
+                # Remove the old (timestamp, ac) from the
+                # sorted list
+                self.sorted_entries.remove((old_timestamp, ac))
+                # Insert the new (timestamp, ac) in the sorted order
+                insort(self.sorted_entries, (timestamp, ac))
+                # Update the ac_to_timestamp dictionary
+                self.ac_to_timestamp[ac] = timestamp
+        else:
+            # Insert the new (timestamp, ac) in the sorted order
+            insort(self.sorted_entries, (timestamp, ac))
+            self.ac_to_timestamp[ac] = timestamp
+
+        # Check size and evict the newest ac(s) if necessary
+        if (len(self.ac_to_timestamp) > self.max_entries):
+            # Pop the newest entry (largest timestamp)
+            _, newest_ac = (self.sorted_entries.pop())
+            del self.ac_to_timestamp[newest_ac]
+
+    def get_n_oldest_timestamp_acs(self, n):
+        oldest_entries = self.sorted_entries[:n]
+        return {
+            'oldest_count': len(oldest_entries),
+            'oldest_entries': [
+                {
+                    'timestamp': entry[0],
+                    'account': entry[1][0],
+                    'container': entry[1][1],
+                }
+                for entry in oldest_entries
+            ],
+        }
+
+    def get_oldest_timestamp(self):
+        if self.sorted_entries:
+            return self.sorted_entries[0][0]
+        return None
+
+    def get_oldest_timestamp_age(self):
+        current_time = time.time()
+        oldest_timestamp = self.get_oldest_timestamp()
+        if oldest_timestamp is not None:
+            return current_time - oldest_timestamp
+        return None
+
+    def reset(self):
+        self.sorted_entries = []
+        self.ac_to_timestamp = {}
+
+    def get_memory_usage(self):
+        return self._get_size(self)
+
+    def _get_size(self, obj, seen=None):
+        if seen is None:
+            seen = set()
+
+        obj_id = id(obj)
+        if obj_id in seen:
+            return 0
+        seen.add(obj_id)
+
+        size = sys.getsizeof(obj)
+
+        if isinstance(obj, dict):
+            size += sum(
+                self._get_size(k, seen) + self._get_size(v, seen)
+                for k, v in obj.items()
+            )
+        elif hasattr(obj, '__dict__'):
+            size += self._get_size(obj.__dict__, seen)
+        elif (
+            hasattr(obj, '__iter__')
+            and not isinstance(obj, (str, bytes, bytearray))
+        ):
+            size += sum(self._get_size(i, seen) for i in obj)
+
+        return size
 
 
 class SweepStats(object):
@@ -229,12 +338,14 @@ class SweepStats(object):
         skips / (skips + successes + failures)
     """
     def __init__(self, errors=0, failures=0, quarantines=0, successes=0,
-                 unlinks=0, redirects=0, skips=0, deferrals=0, drains=0):
+                 unlinks=0, outdated_unlinks=0, redirects=0, skips=0,
+                 deferrals=0, drains=0):
         self.errors = errors
         self.failures = failures
         self.quarantines = quarantines
         self.successes = successes
         self.unlinks = unlinks
+        self.outdated_unlinks = outdated_unlinks
         self.redirects = redirects
         self.skips = skips
         self.deferrals = deferrals
@@ -242,8 +353,9 @@ class SweepStats(object):
 
     def copy(self):
         return type(self)(self.errors, self.failures, self.quarantines,
-                          self.successes, self.unlinks, self.redirects,
-                          self.skips, self.deferrals, self.drains)
+                          self.successes, self.unlinks, self.outdated_unlinks,
+                          self.redirects, self.skips, self.deferrals,
+                          self.drains)
 
     def since(self, other):
         return type(self)(self.errors - other.errors,
@@ -251,6 +363,7 @@ class SweepStats(object):
                           self.quarantines - other.quarantines,
                           self.successes - other.successes,
                           self.unlinks - other.unlinks,
+                          self.outdated_unlinks - other.outdated_unlinks,
                           self.redirects - other.redirects,
                           self.skips - other.skips,
                           self.deferrals - other.deferrals,
@@ -262,6 +375,7 @@ class SweepStats(object):
         self.quarantines = 0
         self.successes = 0
         self.unlinks = 0
+        self.outdated_unlinks = 0
         self.redirects = 0
         self.skips = 0
         self.deferrals = 0
@@ -273,6 +387,7 @@ class SweepStats(object):
             (self.failures, 'failures'),
             (self.quarantines, 'quarantines'),
             (self.unlinks, 'unlinks'),
+            (self.outdated_unlinks, 'outdated_unlinks'),
             (self.errors, 'errors'),
             (self.redirects, 'redirects'),
             (self.skips, 'skips'),
@@ -309,7 +424,8 @@ class ObjectUpdater(Daemon):
         self.interval = float(conf.get('interval', 300))
         self.container_ring = None
         self.concurrency = int(conf.get('concurrency', 8))
-        self.updater_workers = int(conf.get('updater_workers', 1))
+        self.updater_workers = config_positive_int_value(
+            conf.get('updater_workers', 1))
         if 'slowdown' in conf:
             self.logger.warning(
                 'The slowdown option is deprecated in favor of '
@@ -333,7 +449,14 @@ class ObjectUpdater(Daemon):
         self.recon_cache_path = conf.get('recon_cache_path',
                                          DEFAULT_RECON_CACHE_PATH)
         self.rcache = os.path.join(self.recon_cache_path, RECON_OBJECT_FILE)
+        max_entries = config_positive_int_value(
+            conf.get('async_tracker_max_entries', 100)
+        )
+        self.dump_count = config_positive_int_value(
+            conf.get('async_tracker_dump_count', 5)
+        )
         self.stats = SweepStats()
+        self.oldest_async_pendings = OldestAsyncPendingTracker(max_entries)
         self.max_deferred_updates = non_negative_int(
             conf.get('max_deferred_updates', 10000))
         self.begin = time.time()
@@ -355,80 +478,176 @@ class ObjectUpdater(Daemon):
             self.container_ring = Ring(self.swift_dir, ring_name='container')
         return self.container_ring
 
+    def _process_device_in_child(self, dev_path, device):
+        """Process a single device in a forked child process."""
+        signal.signal(signal.SIGTERM, signal.SIG_DFL)
+        os.environ.pop('NOTIFY_SOCKET', None)
+        eventlet_monkey_patch()
+        self.stats.reset()
+        self.oldest_async_pendings.reset()
+        forkbegin = time.time()
+        self.object_sweep(dev_path)
+        elapsed = time.time() - forkbegin
+        self.logger.info(
+            ('Object update sweep of %(device)s '
+                'completed: %(elapsed).02fs, %(stats)s'),
+            {'device': device, 'elapsed': elapsed,
+                'stats': self.stats})
+        self.dump_device_recon(device)
+
+    def _process_devices(self, devices):
+        """Process devices, handling both single and multi-threaded modes."""
+        pids = []
+        # read from container ring to ensure it's fresh
+        self.get_container_ring().get_nodes('')
+        for device in devices:
+            try:
+                dev_path = check_drive(self.devices, device,
+                                       self.mount_check)
+            except ValueError as err:
+                # We don't count this as an error. The occasional
+                # unmounted drive is part of normal cluster operations,
+                # so a simple warning is sufficient.
+                self.logger.warning('Skipping: %s', err)
+                continue
+            while len(pids) >= self.updater_workers:
+                pids.remove(os.wait()[0])
+            pid = os.fork()
+            if pid:
+                pids.append(pid)
+            else:
+                self._process_device_in_child(dev_path, device)
+                sys.exit()
+
+        while pids:
+            pids.remove(os.wait()[0])
+
     def run_forever(self, *args, **kwargs):
         """Run the updater continuously."""
         time.sleep(random() * self.interval)
         while True:
-            self.logger.info('Begin object update sweep')
-            self.begin = time.time()
-            pids = []
-            # read from container ring to ensure it's fresh
-            self.get_container_ring().get_nodes('')
-            for device in self._listdir(self.devices):
-                try:
-                    dev_path = check_drive(self.devices, device,
-                                           self.mount_check)
-                except ValueError as err:
-                    # We don't count this as an error. The occasional
-                    # unmounted drive is part of normal cluster operations,
-                    # so a simple warning is sufficient.
-                    self.logger.warning('Skipping: %s', err)
-                    continue
-                while len(pids) >= self.updater_workers:
-                    pids.remove(os.wait()[0])
-                pid = os.fork()
-                if pid:
-                    pids.append(pid)
-                else:
-                    signal.signal(signal.SIGTERM, signal.SIG_DFL)
-                    os.environ.pop('NOTIFY_SOCKET', None)
-                    eventlet_monkey_patch()
-                    self.stats.reset()
-                    forkbegin = time.time()
-                    self.object_sweep(dev_path)
-                    elapsed = time.time() - forkbegin
-                    self.logger.info(
-                        ('Object update sweep of %(device)s '
-                         'completed: %(elapsed).02fs, %(stats)s'),
-                        {'device': device, 'elapsed': elapsed,
-                         'stats': self.stats})
-                    sys.exit()
-            while pids:
-                pids.remove(os.wait()[0])
-            elapsed = time.time() - self.begin
-            self.logger.info('Object update sweep completed: %.02fs',
-                             elapsed)
-            dump_recon_cache({'object_updater_sweep': elapsed},
-                             self.rcache, self.logger)
+            elapsed = self.run_once(*args, **kwargs)
             if elapsed < self.interval:
                 time.sleep(self.interval - elapsed)
 
     def run_once(self, *args, **kwargs):
         """Run the updater once."""
-        self.logger.info('Begin object update single threaded sweep')
+        self.logger.info('Begin object update sweep of all devices')
         self.begin = time.time()
-        self.stats.reset()
-        for device in self._listdir(self.devices):
-            try:
-                dev_path = check_drive(self.devices, device, self.mount_check)
-            except ValueError as err:
-                # We don't count this as an error. The occasional unmounted
-                # drive is part of normal cluster operations, so a simple
-                # warning is sufficient.
-                self.logger.warning('Skipping: %s', err)
-                continue
-            self.object_sweep(dev_path)
-        elapsed = time.time() - self.begin
+        devices = self._listdir(self.devices)
+        self._process_devices(devices)
+        now = time.time()
+        elapsed = now - self.begin
         self.logger.info(
-            ('Object update single-threaded sweep completed: '
-             '%(elapsed).02fs, %(stats)s'),
-            {'elapsed': elapsed, 'stats': self.stats})
-        dump_recon_cache({'object_updater_sweep': elapsed},
-                         self.rcache, self.logger)
+            ('Object update sweep of all devices completed: '
+             '%(elapsed).02fs'),
+            {'elapsed': elapsed})
+        self.aggregate_and_dump_recon(devices, elapsed, now)
+        return elapsed
+
+    def _gather_recon_stats(self):
+        """Gather stats for device recon dumps."""
+        stats = {
+            'failures_oldest_timestamp':
+                self.oldest_async_pendings.get_oldest_timestamp(),
+            'failures_oldest_timestamp_age':
+                self.oldest_async_pendings.get_oldest_timestamp_age(),
+            'failures_account_container_count':
+                len(self.oldest_async_pendings.ac_to_timestamp),
+            'failures_oldest_timestamp_account_containers':
+                self.oldest_async_pendings.get_n_oldest_timestamp_acs(
+                    self.dump_count),
+            'tracker_memory_usage':
+                self.oldest_async_pendings.get_memory_usage(),
+        }
+        return stats
+
+    def dump_device_recon(self, device):
+        """Dump recon stats for a single device."""
+        disk_recon_stats = self._gather_recon_stats()
+        dump_recon_cache(
+            {'object_updater_per_device': {device: disk_recon_stats}},
+            self.rcache,
+            self.logger,
+        )
+
+    def aggregate_and_dump_recon(self, devices, elapsed, now):
+        """
+        Aggregate recon stats across devices and dump the result to the
+        recon cache.
+        """
+        recon_cache = load_recon_cache(self.rcache)
+        device_stats = recon_cache.get('object_updater_per_device', {})
+        if not isinstance(device_stats, dict):
+            raise TypeError('object_updater_per_device must be a dict')
+        devices_to_remove = set(device_stats) - set(devices)
+        device_stats = {dev: device_stats.get(dev) or {}
+                        for dev in devices}
+
+        aggregated_oldest_entries = []
+
+        for stats in device_stats.values():
+            container_data = stats.get(
+                'failures_oldest_timestamp_account_containers', {})
+            aggregated_oldest_entries.extend(container_data.get(
+                'oldest_entries', []))
+        aggregated_oldest_entries.sort(key=lambda x: x['timestamp'])
+        aggregated_oldest_entries = aggregated_oldest_entries[:self.dump_count]
+        aggregated_oldest_count = len(aggregated_oldest_entries)
+
+        aggregated_stats = {
+            'failures_account_container_count': max(
+                list(
+                    stats.get('failures_account_container_count', 0)
+                    for stats in device_stats.values()
+                )
+                or [0],
+            ),
+            'tracker_memory_usage': (
+                float(sum(
+                    stats.get('tracker_memory_usage', 0)
+                    for stats in device_stats.values()
+                ))
+                / float(len(device_stats))
+            )
+            * max(1, min(self.updater_workers, len(device_stats)))
+            if device_stats
+            else 0,
+            'failures_oldest_timestamp': min(
+                list(filter(lambda x: x is not None,
+                            [stats.get('failures_oldest_timestamp')
+                             for stats in device_stats.values()]
+                            )) or [None],
+            ),
+            'failures_oldest_timestamp_age': max(
+                list(filter(lambda x: x is not None,
+                            [stats.get('failures_oldest_timestamp_age')
+                             for stats in device_stats.values()]
+                            )) or [None],
+            ),
+            'failures_oldest_timestamp_account_containers': {
+                'oldest_count': aggregated_oldest_count,
+                'oldest_entries': aggregated_oldest_entries,
+            },
+        }
+
+        recon_dump = {
+            'object_updater_sweep': elapsed,
+            'object_updater_stats': aggregated_stats,
+            'object_updater_last': now
+        }
+        if devices_to_remove:
+            recon_dump['object_updater_per_device'] = {
+                d: {} for d in devices_to_remove}
+        dump_recon_cache(
+            recon_dump,
+            self.rcache,
+            self.logger,
+        )
 
     def _load_update(self, device, update_path):
         try:
-            return pickle.load(open(update_path, 'rb'))
+            return pickle.load(open(update_path, 'rb'))  # nosec: B301
         except Exception as e:
             if getattr(e, 'errno', None) == errno.ENOENT:
                 return
@@ -520,8 +739,8 @@ class ObjectUpdater(Daemon):
                     #
                     # This way, our caller only gets useful async_pendings.
                     if obj_hash == last_obj_hash:
-                        self.stats.unlinks += 1
-                        self.logger.increment('unlinks')
+                        self.stats.outdated_unlinks += 1
+                        self.logger.increment('outdated_unlinks')
                         try:
                             os.unlink(update_path)
                         except OSError as e:
@@ -583,7 +802,9 @@ class ObjectUpdater(Daemon):
              'in %(elapsed).02fs seconds:, '
              '%(successes)d successes, %(failures)d failures, '
              '%(quarantines)d quarantines, '
-             '%(unlinks)d unlinks, %(errors)d errors, '
+             '%(unlinks)d unlinks, '
+             '%(outdated_unlinks)d outdated_unlinks, '
+             '%(errors)d errors, '
              '%(redirects)d redirects, '
              '%(skips)d skips, '
              '%(deferrals)d deferrals, '
@@ -596,6 +817,7 @@ class ObjectUpdater(Daemon):
              'failures': sweep_totals.failures,
              'quarantines': sweep_totals.quarantines,
              'unlinks': sweep_totals.unlinks,
+             'outdated_unlinks': sweep_totals.outdated_unlinks,
              'errors': sweep_totals.errors,
              'redirects': sweep_totals.redirects,
              'skips': sweep_totals.skips,
@@ -625,9 +847,9 @@ class ObjectUpdater(Daemon):
             headers_out.setdefault('X-Backend-Accept-Quoted-Location', 'true')
             acct, cont = split_update_path(update)
             part, nodes = self.get_container_ring().get_nodes(acct, cont)
-            obj = '/%s/%s/%s' % (acct, cont, update['obj'])
+            path = '/%s/%s/%s' % (acct, cont, update['obj'])
             events = [spawn(self.object_update,
-                            node, part, update['op'], obj, headers_out)
+                            node, part, update['op'], path, headers_out)
                       for node in nodes if node['id'] not in successes]
             success = True
             new_successes = rewrite_pickle = False
@@ -646,8 +868,8 @@ class ObjectUpdater(Daemon):
             if success:
                 self.stats.successes += 1
                 self.logger.increment('successes')
-                self.logger.debug('Update sent for %(obj)s %(path)s',
-                                  {'obj': obj, 'path': update_path})
+                self.logger.debug('Update sent for %(path)s %(update_path)s',
+                                  {'path': path, 'update_path': update_path})
                 self.stats.unlinks += 1
                 self.logger.increment('unlinks')
                 os.unlink(update_path)
@@ -673,15 +895,19 @@ class ObjectUpdater(Daemon):
                 self.stats.redirects += 1
                 self.logger.increment("redirects")
                 self.logger.debug(
-                    'Update redirected for %(obj)s %(path)s to %(shard)s',
-                    {'obj': obj, 'path': update_path,
+                    'Update redirected for %(path)s %(update_path)s to '
+                    '%(shard)s',
+                    {'path': path, 'update_path': update_path,
                      'shard': update['container_path']})
                 rewrite_pickle = True
             else:
                 self.stats.failures += 1
                 self.logger.increment('failures')
-                self.logger.debug('Update failed for %(obj)s %(path)s',
-                                  {'obj': obj, 'path': update_path})
+                self.logger.debug('Update failed for %(path)s %(update_path)s',
+                                  {'path': path, 'update_path': update_path})
+                self.oldest_async_pendings.add_update(
+                    acct, cont, kwargs['timestamp']
+                )
                 if new_successes:
                     update['successes'] = successes
                     rewrite_pickle = True
@@ -696,14 +922,14 @@ class ObjectUpdater(Daemon):
             write_pickle(update, update_path, os.path.join(
                 device, get_tmp_dir(policy)))
 
-    def object_update(self, node, part, op, obj, headers_out):
+    def object_update(self, node, part, op, path, headers_out):
         """
         Perform the object update to the container
 
         :param node: node dictionary from the container ring
         :param part: partition that holds the container
         :param op: operation performed (ex: 'PUT' or 'DELETE')
-        :param obj: object name being updated
+        :param path: /<acct>/<cont>/<obj> path being updated
         :param headers_out: headers to send with the update
         :return: a tuple of (``success``, ``node_id``, ``redirect``)
             where ``success`` is True if the update succeeded, ``node_id`` is
@@ -718,7 +944,7 @@ class ObjectUpdater(Daemon):
             with ConnectionTimeout(self.conn_timeout):
                 conn = http_connect(
                     node['replication_ip'], node['replication_port'],
-                    node['device'], part, op, obj, headers_out)
+                    node['device'], part, op, path, headers_out)
             with Timeout(self.node_timeout):
                 resp = conn.getresponse()
                 resp.read()
@@ -730,7 +956,7 @@ class ObjectUpdater(Daemon):
                 except ValueError as err:
                     self.logger.error(
                         'Container update failed for %r; problem with '
-                        'redirect location: %s' % (obj, err))
+                        'redirect location: %s' % (path, err))
 
             success = is_success(status)
             if not success:
