@@ -36,9 +36,11 @@ from swift.common.utils import Watchdog, get_logger, \
     get_remote_client, split_path, config_true_value, generate_trans_id, \
     affinity_key_function, affinity_locality_predicate, list_from_csv, \
     parse_prefixed_conf, config_auto_int_value, node_to_string, \
-    config_request_node_count_value, config_percent_value, cap_length
+    config_request_node_count_value, config_percent_value, cap_length, \
+    parse_options, non_negative_int, config_positive_float_value
 from swift.common.registry import register_swift_info
 from swift.common.constraints import check_utf8, valid_api_version
+from swift.common.statsd_client import get_labeled_statsd_client
 from swift.proxy.controllers import AccountController, ContainerController, \
     ObjectControllerRouter, InfoController
 from swift.proxy.controllers.base import get_container_info, \
@@ -49,6 +51,11 @@ from swift.common.swob import HTTPBadRequest, HTTPForbidden, \
     HTTPServerError, HTTPException, Request, HTTPServiceUnavailable, \
     wsgi_to_str
 from swift.common.exceptions import APIVersionError
+from swift.common.wsgi import run_wsgi
+from swift.obj import expirer
+
+DEFAULT_NAMESPACE_AVG_BACKEND_FETCH_TIME = 0.3  # seconds
+DEFAULT_NAMESPACE_CACHE_TOKENS_PER_SESSION = 3  # 3 tokens per session
 
 
 # List of entry points for mandatory middlewares.
@@ -193,7 +200,7 @@ class Application(object):
     """WSGI application for the proxy server."""
 
     def __init__(self, conf, logger=None, account_ring=None,
-                 container_ring=None):
+                 container_ring=None, statsd=None):
         # This is for the sake of tests which instantiate an Application
         # directly rather than via loadapp().
         self._pipeline_final_app = self
@@ -205,6 +212,7 @@ class Application(object):
                                      statsd_tail_prefix='proxy-server')
         else:
             self.logger = logger
+        self.statsd = statsd or get_labeled_statsd_client(conf, self.logger)
         self.backend_user_agent = 'proxy-server %s' % os.getpid()
 
         swift_dir = conf.get('swift_dir', '/etc/swift')
@@ -246,6 +254,20 @@ class Application(object):
                 'container_listing_shard_ranges_skip_cache_pct', 0))
         self.account_existence_skip_cache = config_percent_value(
             conf.get('account_existence_skip_cache_pct', 0))
+        self.namespace_avg_backend_fetch_time = \
+            config_positive_float_value(
+                conf.get(
+                    'namespace_avg_backend_fetch_time',
+                    DEFAULT_NAMESPACE_AVG_BACKEND_FETCH_TIME
+                )
+            )
+        self.namespace_cache_tokens_per_session = \
+            non_negative_int(
+                conf.get(
+                    'namespace_cache_tokens_per_session',
+                    DEFAULT_NAMESPACE_CACHE_TOKENS_PER_SESSION
+                )
+            )
         self.allow_account_management = \
             config_true_value(conf.get('allow_account_management', 'no'))
         self.container_ring = container_ring or Ring(swift_dir,
@@ -262,10 +284,8 @@ class Application(object):
             config_true_value(conf.get('account_autocreate', 'no'))
         self.auto_create_account_prefix = \
             constraints.AUTO_CREATE_ACCOUNT_PREFIX
-        self.expiring_objects_account = self.auto_create_account_prefix + \
-            (conf.get('expiring_objects_account_name') or 'expiring_objects')
-        self.expiring_objects_container_divisor = \
-            int(conf.get('expiring_objects_container_divisor') or 86400)
+        self.expirer_config = expirer.ExpirerConfig(
+            conf, container_ring=self.container_ring, logger=self.logger)
         self.max_containers_per_account = \
             int(conf.get('max_containers_per_account') or 0)
         self.max_containers_whitelist = [
@@ -286,6 +306,8 @@ class Application(object):
             if a.strip()]
         self.strict_cors_mode = config_true_value(
             conf.get('strict_cors_mode', 't'))
+        self.allow_open_expired = config_true_value(
+            conf.get('allow_open_expired', 'f'))
         self.node_timings = {}
         self.timing_expiry = int(conf.get('timing_expiry', 300))
         value = conf.get('request_node_count', '2 * replicas')
@@ -347,6 +369,7 @@ class Application(object):
             policies=POLICIES.get_policy_info(),
             allow_account_management=self.allow_account_management,
             account_autocreate=self.account_autocreate,
+            allow_open_expired=self.allow_open_expired,
             **constraints.EFFECTIVE_CONSTRAINTS)
         self.watchdog = Watchdog()
         self.watchdog.spawn()
@@ -815,3 +838,12 @@ def app_factory(global_conf, **local_conf):
     app = Application(conf)
     app.check_config()
     return app
+
+
+def main():
+    conf_file, options = parse_options(test_config=True)
+    sys.exit(run_wsgi(conf_file, 'proxy-server', **options))
+
+
+if __name__ == '__main__':
+    main()
