@@ -13,12 +13,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import print_function
 import sys
-from contextlib import contextmanager
 
 import os
-from six import reraise
 
 from unittest.util import safe_repr
 
@@ -38,13 +35,10 @@ warnings.filterwarnings('ignore', message=(
 
 import unittest
 
-if sys.version_info < (3, 2):
-    unittest.TestCase.assertRaisesRegex = unittest.TestCase.assertRaisesRegexp
-    unittest.TestCase.assertRegex = unittest.TestCase.assertRegexpMatches
-
 from eventlet.green import socket
 
 from swift.common.utils import readconf
+from swift.common.utils.timestamp import Timestamp
 
 
 # Work around what seems to be a Python bug.
@@ -93,29 +87,6 @@ def listen_zero():
     return sock
 
 
-@contextmanager
-def annotate_failure(msg):
-    """
-    Catch AssertionError and annotate it with a message. Useful when making
-    assertions in a loop where the message can indicate the loop index or
-    richer context about the failure.
-
-    :param msg: A message to be prefixed to the AssertionError message.
-    """
-    try:
-        yield
-    except AssertionError as err:
-        err_typ, err_val, err_tb = sys.exc_info()
-        if err_val.args:
-            msg = '%s Failed with %s' % (msg, err_val.args[0])
-            err_val.args = (msg, ) + err_val.args[1:]
-        else:
-            # workaround for some IDE's raising custom AssertionErrors
-            err_val = '%s Failed with %s' % (msg, err)
-            err_typ = AssertionError
-        reraise(err_typ, err_val, err_tb)
-
-
 class BaseTestCase(unittest.TestCase):
     def _assertDictContainsSubset(self, subset, dictionary, msg=None):
         """Checks whether dictionary is a superset of subset."""
@@ -145,3 +116,15 @@ class BaseTestCase(unittest.TestCase):
             standardMsg += 'Mismatched values: %s' % ','.join(mismatched)
 
         self.fail(self._formatMessage(msg, standardMsg))
+
+    def assert_valid_timestamp(self, timestamp):
+        """
+        Helper that asserts the given timestamp is a valid Timestamp
+        representation.
+
+        :param timestamp: a string or instance of Timestamp
+        """
+        try:
+            timestamp = Timestamp(timestamp)
+        except (ValueError, TypeError) as e:
+            self.fail('Invalid timestamp (%r): %r' % (timestamp, e))
