@@ -171,6 +171,13 @@ class BaseBrain(object, metaclass=meta_command):
         self.client.put_object(self.container_name, self.object_name,
                                headers=headers, contents=contents)
 
+    def post_object(self, headers=None):
+        """
+        issue POST for test object
+        """
+        return self.client.post_object(
+            self.container_name, self.object_name, headers=headers)
+
     @command
     def delete_object(self):
         """
@@ -178,6 +185,13 @@ class BaseBrain(object, metaclass=meta_command):
         """
         self.client.delete_object(self.container_name, self.object_name)
 
+    @command
+    def head_object(self):
+        """
+        issue HEAD for test object
+        """
+        return self.client.head_object(self.container_name, self.object_name)
+
     @command
     def get_object(self):
         """
@@ -204,6 +218,10 @@ class PublicBrainClient(object):
         return client.get_container(self.url, self.token, container_name,
                                     headers=headers, query_string=query_string)
 
+    def post_object(self, container_name, object_name, headers=None):
+        return client.post_object(self.url, self.token, container_name,
+                                  object_name, headers=headers)
+
     def delete_container(self, container_name):
         return client.delete_container(self.url, self.token, container_name)
 
