@@ -20,7 +20,7 @@ import shutil
 import tempfile
 import unittest
 
-from mock import patch, call, DEFAULT
+from unittest.mock import patch, call, DEFAULT
 import eventlet
 
 from swift.account import reaper
@@ -59,7 +59,7 @@ class FakeAccountBroker(object):
                 kwargs, ))
         for cont in self.containers:
             if cont > marker:
-                yield cont, None, None, None, None
+                yield cont, None, None, None, None, None
             limit -= 1
             if limit <= 0:
                 break
@@ -308,8 +308,9 @@ class TestReaper(unittest.TestCase):
         for policy in POLICIES:
             r.reset_stats()
             with patch(mock_path) as fake_direct_delete:
-                with patch('swift.common.utils.Timestamp.now') as mock_now:
-                    mock_now.return_value = Timestamp(1429117638.86767)
+                ts_now = Timestamp(1429117638.86767)
+                with patch('swift.common.utils.Timestamp.now',
+                           return_value=ts_now) as mock_now:
                     r.reap_object('a', 'c', 'partition', cont_nodes, 'o',
                                   policy.idx)
                     mock_now.assert_called_once_with()
@@ -323,7 +324,7 @@ class TestReaper(unittest.TestCase):
                             'X-Container-Partition': 'partition',
                             'X-Container-Device': device,
                             'X-Backend-Storage-Policy-Index': policy.idx,
-                            'X-Timestamp': '1429117638.86767',
+                            'X-Timestamp': ts_now.internal,
                             'x-backend-use-replication-network': 'true',
                         }
                         ring = r.get_object_ring(policy.idx)
@@ -415,9 +416,10 @@ class TestReaper(unittest.TestCase):
                 return headers, obj_list
 
             mocks['direct_get_container'].side_effect = fake_get_container
-            with patch('swift.common.utils.Timestamp.now') as mock_now:
-                mock_now.side_effect = [Timestamp(1429117638.86767),
-                                        Timestamp(1429117639.67676)]
+            mock_timestamps = [Timestamp(1429117638.86767),
+                               Timestamp(1429117639.67676)]
+            with patch('swift.common.utils.Timestamp.now',
+                       side_effect=mock_timestamps):
                 r.reap_container('a', 'partition', acc_nodes, 'c')
 
             # verify calls to direct_delete_object
@@ -428,9 +430,8 @@ class TestReaper(unittest.TestCase):
                 self.assertEqual(kwargs['headers']
                                  ['X-Backend-Storage-Policy-Index'],
                                  policy.idx)
-                self.assertEqual(kwargs['headers']
-                                 ['X-Timestamp'],
-                                 '1429117638.86767')
+                self.assertEqual(kwargs['headers']['X-Timestamp'],
+                                 mock_timestamps[0].internal)
 
             # verify calls to direct_delete_container
             self.assertEqual(mocks['direct_delete_container'].call_count, 3)
@@ -444,7 +445,7 @@ class TestReaper(unittest.TestCase):
                     'X-Account-Partition': 'partition',
                     'X-Account-Device': device,
                     'X-Account-Override-Deleted': 'yes',
-                    'X-Timestamp': '1429117639.67676',
+                    'X-Timestamp': mock_timestamps[1].internal,
                     'x-backend-use-replication-network': 'true',
                 }
                 ring = r.get_object_ring(policy.idx)
@@ -470,7 +471,7 @@ class TestReaper(unittest.TestCase):
                       self.fake_reap_object):
             r.reap_container('a', 'partition', acc_nodes, 'c')
         self.assertEqual(
-            r.logger.statsd_client.get_increment_counts()['return_codes.4'], 1)
+            r.logger.statsd_client.get_stats_counts()['return_codes.4'], 1)
         self.assertEqual(r.stats_containers_deleted, 1)
 
     def test_reap_container_partial_fail(self):
@@ -490,7 +491,7 @@ class TestReaper(unittest.TestCase):
                       self.fake_reap_object):
             r.reap_container('a', 'partition', acc_nodes, 'c')
         self.assertEqual(
-            r.logger.statsd_client.get_increment_counts()['return_codes.4'], 4)
+            r.logger.statsd_client.get_stats_counts()['return_codes.4'], 4)
         self.assertEqual(r.stats_containers_possibly_remaining, 1)
 
     def test_reap_container_full_fail(self):
@@ -510,7 +511,7 @@ class TestReaper(unittest.TestCase):
                       self.fake_reap_object):
             r.reap_container('a', 'partition', acc_nodes, 'c')
         self.assertEqual(
-            r.logger.statsd_client.get_increment_counts()['return_codes.4'], 5)
+            r.logger.statsd_client.get_stats_counts()['return_codes.4'], 5)
         self.assertEqual(r.stats_containers_remaining, 1)
 
     def test_reap_container_get_object_timeout(self):
@@ -735,7 +736,7 @@ class TestReaper(unittest.TestCase):
                 if container in self.containers_yielded:
                     continue
 
-                yield container, None, None, None, None
+                yield container, None, None, None, None, None
                 self.containers_yielded.append(container)
 
         def fake_reap_container(self, account, account_partition,
