@@ -14,11 +14,10 @@
 # limitations under the License.
 
 import unittest
-import mock
+from unittest import mock
 from hashlib import sha256
 
-import six
-from six.moves.urllib.parse import quote, parse_qsl
+from urllib.parse import quote, parse_qsl
 
 from swift.common import swob
 from swift.common.middleware.proxy_logging import ProxyLoggingMiddleware
@@ -282,7 +281,12 @@ class BaseS3ApiBucket(object):
         self.assertEqual(body, b'')
         self.assertEqual(status.split()[0], '200')
         self.assertEqual(headers['Location'], '/bucket')
+        method, path, sw_headers = self.swift.calls_with_headers[0]
+        self.assertEqual('PUT', method)
+        self.assertEqual('/v1/AUTH_test/bucket', path)
+        self.assertNotIn('X-Timestamp', sw_headers)
 
+    def test_bucket_PUT_chunked_transfer(self):
         # Apparently some clients will include a chunked transfer-encoding
         # even with no body
         req = Request.blank('/bucket',
@@ -294,7 +298,12 @@ class BaseS3ApiBucket(object):
         self.assertEqual(body, b'')
         self.assertEqual(status.split()[0], '200')
         self.assertEqual(headers['Location'], '/bucket')
+        method, path, sw_headers = self.swift.calls_with_headers[0]
+        self.assertEqual('PUT', method)
+        self.assertEqual('/v1/AUTH_test/bucket', path)
+        self.assertNotIn('X-Timestamp', sw_headers)
 
+    def test_bucket_PUT_unreadable_input(self):
         with UnreadableInput(self) as fake_input:
             req = Request.blank(
                 '/bucket',
@@ -306,6 +315,10 @@ class BaseS3ApiBucket(object):
         self.assertEqual(body, b'')
         self.assertEqual(status.split()[0], '200')
         self.assertEqual(headers['Location'], '/bucket')
+        method, path, sw_headers = self.swift.calls_with_headers[0]
+        self.assertEqual('PUT', method)
+        self.assertEqual('/v1/AUTH_test/bucket', path)
+        self.assertNotIn('X-Timestamp', sw_headers)
 
     def test_bucket_PUT_with_location(self):
         self._test_bucket_PUT_with_location('CreateBucketConfiguration')
@@ -407,6 +420,10 @@ class BaseS3ApiBucket(object):
                                      'Date': self.get_date_header()})
         status, headers, body = self.call_s3api(req)
         self.assertEqual(status.split()[0], '204')
+        method, path, sw_headers = self.swift.calls_with_headers[-1]
+        self.assertEqual('DELETE', method)
+        self.assertEqual('/v1/AUTH_test/bucket', path)
+        self.assertNotIn('X-Timestamp', sw_headers)
 
     def test_bucket_DELETE_with_empty_versioning(self):
         self.swift.register('HEAD', '/v1/AUTH_test/bucket+versioning',
@@ -472,7 +489,8 @@ class TestS3ApiBucketNoACL(BaseS3ApiBucket, S3ApiTestCase):
         self.assertEqual(1, len(access_lines))
         parts = access_lines[0].split()
         self.assertEqual(' '.join(parts[3:7]), 'HEAD /junk HTTP/1.0 200')
-        self.assertEqual(parts[-1], str(bucket_policy_index))
+        self.assertEqual(parts[-2], str(bucket_policy_index))
+        self.assertEqual(parts[-1], 'test:tester')  # access_user_id
 
     def test_bucket_HEAD_policy_index_logging(self):
         self._do_test_bucket_HEAD_policy_index_logging(0)
@@ -535,7 +553,7 @@ class TestS3ApiBucketNoACL(BaseS3ApiBucket, S3ApiTestCase):
             self.assertEqual('2011-01-05T02:19:15.000Z',
                              o.find('./LastModified').text)
         expected = [
-            (i[0].encode('utf-8') if six.PY2 else i[0],
+            (i[0],
              PFS_ETAG if i[0] == 'pfs-obj' else
              '"0-N"' if i[0] == 'slo' else '"0"')
             for i in self.objects
@@ -646,12 +664,8 @@ class TestS3ApiBucketNoACL(BaseS3ApiBucket, S3ApiTestCase):
         status, headers, body = self.call_s3api(req)
         elem = fromstring(body, 'ListBucketResult')
         self.assertEqual(elem.find('./IsTruncated').text, 'true')
-        if six.PY2:
-            self.assertEqual(elem.find('./NextMarker').text,
-                             u'but-\u062a/'.encode('utf-8'))
-        else:
-            self.assertEqual(elem.find('./NextMarker').text,
-                             u'but-\u062a/')
+        self.assertEqual(elem.find('./NextMarker').text,
+                         u'but-\u062a/')
 
     def test_bucket_GET_is_truncated_url_encoded(self):
         bucket_name = 'junk'
@@ -1023,10 +1037,7 @@ class TestS3ApiBucketNoACL(BaseS3ApiBucket, S3ApiTestCase):
         self.assertEqual(elem.findall('./DeleteMarker'), [])
         versions = elem.findall('./Version')
         objects = list(self.objects)
-        if six.PY2:
-            expected = [v[0].encode('utf-8') for v in objects]
-        else:
-            expected = [v[0] for v in objects]
+        expected = [v[0] for v in objects]
         self.assertEqual([v.find('./Key').text for v in versions], expected)
         self.assertEqual([v.find('./IsLatest').text for v in versions],
                          ['true' for v in objects])
@@ -1067,8 +1078,6 @@ class TestS3ApiBucketNoACL(BaseS3ApiBucket, S3ApiTestCase):
         expected = []
         for o in self.objects_list:
             name = o['name']
-            if six.PY2:
-                name = name.encode('utf8')
             expected.append((name, 'true', 'null'))
             if name == 'rose':
                 expected.append((name, 'false', '1'))
@@ -1097,8 +1106,6 @@ class TestS3ApiBucketNoACL(BaseS3ApiBucket, S3ApiTestCase):
         expected = []
         for o in self.objects_list[:5]:
             name = o['name']
-            if six.PY2:
-                name = name.encode('utf8')
             expected.append((name, 'true', 'null'))
             if name == 'rose':
                 expected.append((name, 'false', '1'))
@@ -1265,7 +1272,7 @@ class TestS3ApiBucketNoACL(BaseS3ApiBucket, S3ApiTestCase):
         self.assertEqual(expected, discovered)
         expected = [
             ('lily', 'true', 'null'),
-            (b'lily-\xd8\xaa', 'true', 'null'),
+            ('lily-\u062a', 'true', 'null'),
             ('mu', 'true', 'null'),
             ('pfs-obj', 'true', 'null'),
             ('rose', 'true', 'null'),
@@ -1275,10 +1282,6 @@ class TestS3ApiBucketNoACL(BaseS3ApiBucket, S3ApiTestCase):
             ('with space', 'true', 'null'),
             ('with%20space', 'true', 'null'),
         ]
-        if not six.PY2:
-            item = list(expected[1])
-            item[0] = item[0].decode('utf8')
-            expected[1] = tuple(item)
 
         discovered = [
             tuple(e.find('./%s' % key).text for key in (
@@ -1510,9 +1513,16 @@ class TestS3ApiBucketNoACL(BaseS3ApiBucket, S3ApiTestCase):
                             environ={'REQUEST_METHOD': 'PUT'},
                             headers=headers,
                             body=req_body)
+
+        # Test V4 bucket creation and access_user_id setting
         status, headers, body = self.call_s3api(req)
         self.assertEqual(status.split()[0], '200', body)
 
+        # Verify access_user_id is set correctly in environ for V4 bucket
+        # creation
+        self.assertEqual(req.environ['swift.access_logging']['user_id'],
+                         'test:tester')
+
     def test_bucket_PUT_v4_with_body_bad_hash(self):
         elem = Element('CreateBucketConfiguration')
         SubElement(elem, 'LocationConstraint').text = self.s3api.conf.location
