@@ -17,11 +17,12 @@ import functools
 import json
 import os
 import time
-import mock
 import unittest
 from swift.common import swob, utils, registry
 from swift.common.middleware import versioned_writes, copy
-from swift.common.swob import Request
+from swift.common.swob import Request, date_header_format
+from swift.common.utils import Timestamp
+from test.unit import mock_timestamp_now, make_timestamp_iter
 from test.unit.common.middleware import helpers
 
 
@@ -394,7 +395,7 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
         self.assertEqual(status, '201 Created')
         self.assertEqual(len(self.authorized), 1)
         self.assertRequestEqual(req, self.authorized[0])
-        called_method = [method for (method, path, hdrs) in self.app._calls]
+        called_method = [call.method for call in self.app.call_list]
         self.assertNotIn('GET', called_method)
 
     def test_put_request_is_dlo_manifest_with_container_config_true(self):
@@ -424,7 +425,7 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
             ('PUT', '/v1/a/c/o'),
         ], self.app.calls)
         self.assertIn('x-object-manifest',
-                      self.app.calls_with_headers[2].headers)
+                      self.app.call_list[2].headers)
 
     def test_put_version_is_dlo_manifest_with_container_config_true(self):
         self.app.register('GET', '/v1/a/c/o', swob.HTTPOk,
@@ -456,7 +457,7 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
             ('PUT', '/v1/a/c/o'),
         ], self.app.calls)
         self.assertIn('x-object-manifest',
-                      self.app.calls_with_headers[1].headers)
+                      self.app.call_list[1].headers)
 
     def test_delete_object_no_versioning_with_container_config_true(self):
         # set False to versions_write obviously and expect no GET versioning
@@ -472,8 +473,7 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
         self.assertEqual(status, '204 No Content')
         self.assertEqual(len(self.authorized), 1)
         self.assertRequestEqual(req, self.authorized[0])
-        called_method = \
-            [method for (method, path, rheaders) in self.app._calls]
+        called_method = [call.method for call in self.app.call_list]
         self.assertNotIn('PUT', called_method)
         self.assertNotIn('GET', called_method)
         self.assertEqual(1, self.app.call_count)
@@ -724,17 +724,17 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
         self.assertRequestEqual(req, self.authorized[0])
         self.assertRequestEqual(req, self.authorized[1])
 
-        calls = self.app.calls_with_headers
+        calls = self.app.call_list
         self.assertEqual(['GET', 'HEAD', 'DELETE'],
                          [c.method for c in calls])
 
         self.assertIn('X-Newest', calls[1].headers)
         self.assertEqual('True', calls[1].headers['X-Newest'])
 
-        method, path, req_headers = calls.pop()
-        self.assertTrue(path.startswith('/v1/a/c/o'))
+        call = calls.pop()
+        self.assertTrue(call.path.startswith('/v1/a/c/o'))
         # Since we're deleting the original, this *should* still be present:
-        self.assertEqual('1', req_headers.get('X-If-Delete-At'))
+        self.assertEqual('1', call.headers.get('X-If-Delete-At'))
 
     def test_delete_latest_version_is_marker_success(self):
         # Test popping a delete marker off the stack. So, there's data in the
@@ -836,23 +836,22 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
         self.assertRequestEqual(req, self.authorized[0])
 
         # check that X-If-Delete-At was removed from DELETE request
-        calls = self.app.calls_with_headers
+        calls = self.app.call_list
         self.assertEqual(['GET', 'HEAD', 'DELETE'],
                          [c.method for c in calls])
 
-        method, path, req_headers = calls.pop()
-        self.assertTrue(path.startswith('/v1/a/ver_cont/001o/3'))
-        self.assertNotIn('x-if-delete-at', [h.lower() for h in req_headers])
+        call = calls.pop()
+        self.assertTrue(call.path.startswith('/v1/a/ver_cont/001o/3'))
+        self.assertNotIn('x-if-delete-at', [h.lower() for h in call.headers])
 
-    @mock.patch('swift.common.middleware.versioned_writes.legacy.time.time',
-                return_value=1234)
-    def test_history_delete_marker_no_object_success(self, mock_time):
+    def test_history_delete_marker_no_object_success(self):
+        ts_now = Timestamp.now()
         self.app.register(
             'GET', '/v1/a/c/o', swob.HTTPNotFound,
             {}, 'passed')
         self.app.register(
-            'PUT', '/v1/a/ver_cont/001o/0000001234.00000', swob.HTTPCreated,
-            {}, 'passed')
+            'PUT', '/v1/a/ver_cont/001o/%s' % ts_now.internal,
+            swob.HTTPCreated, {}, 'passed')
         self.app.register(
             'DELETE', '/v1/a/c/o', swob.HTTPNotFound, {}, None)
 
@@ -862,21 +861,21 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
             '/v1/a/c/o',
             environ={'REQUEST_METHOD': 'DELETE', 'swift.cache': cache,
                      'CONTENT_LENGTH': '0'})
-        status, headers, body = self.call_vw(req)
+        with mock_timestamp_now(ts_now):
+            status, headers, body = self.call_vw(req)
         self.assertEqual(status, '404 Not Found')
         self.assertEqual(len(self.authorized), 2)
 
         req.environ['REQUEST_METHOD'] = 'PUT'
         self.assertRequestEqual(req, self.authorized[0])
 
-        calls = self.app.calls_with_headers
+        calls = self.app.call_list
         self.assertEqual(['GET', 'PUT', 'DELETE'], [c.method for c in calls])
         self.assertEqual('application/x-deleted;swift_versions_deleted=1',
                          calls[1].headers.get('Content-Type'))
 
-    @mock.patch('swift.common.middleware.versioned_writes.legacy.time.time',
-                return_value=123456789.54321)
-    def test_history_delete_marker_over_object_success(self, mock_time):
+    def test_history_delete_marker_over_object_success(self):
+        ts_now = Timestamp.now()
         self.app.register(
             'GET', '/v1/a/c/o', swob.HTTPOk,
             {'last-modified': 'Wed, 19 Nov 2014 18:19:02 GMT'}, 'passed')
@@ -884,8 +883,8 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
             'PUT', '/v1/a/ver_cont/001o/1416421142.00000', swob.HTTPCreated,
             {}, 'passed')
         self.app.register(
-            'PUT', '/v1/a/ver_cont/001o/0123456789.54321', swob.HTTPCreated,
-            {}, 'passed')
+            'PUT', '/v1/a/ver_cont/001o/%s' % ts_now.internal,
+            swob.HTTPCreated, {}, 'passed')
         self.app.register(
             'DELETE', '/v1/a/c/o', swob.HTTPNoContent, {}, None)
 
@@ -895,7 +894,8 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
             '/v1/a/c/o',
             environ={'REQUEST_METHOD': 'DELETE', 'swift.cache': cache,
                      'CONTENT_LENGTH': '0'})
-        status, headers, body = self.call_vw(req)
+        with mock_timestamp_now(ts_now):
+            status, headers, body = self.call_vw(req)
         self.assertEqual(status, '204 No Content')
         self.assertEqual(b'', body)
         self.assertEqual(len(self.authorized), 2)
@@ -903,7 +903,7 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
         req.environ['REQUEST_METHOD'] = 'PUT'
         self.assertRequestEqual(req, self.authorized[0])
 
-        calls = self.app.calls_with_headers
+        calls = self.app.call_list
         self.assertEqual(['GET', 'PUT', 'PUT', 'DELETE'],
                          [c.method for c in calls])
         self.assertEqual('/v1/a/ver_cont/001o/1416421142.00000',
@@ -1462,6 +1462,42 @@ class VersionedWritesCopyingTestCase(VersionedWritesBaseTestCase):
         self.assertEqual('/v1/a/tgt_cont/tgt_obj', self.authorized[1].path)
         self.assertEqual(4, self.app.call_count)
 
+    def test_copy_new_version_x_timestamp(self):
+        # existing object should be moved to versions container
+        ts_iter = make_timestamp_iter()
+        ts_then, ts_now = next(ts_iter), next(ts_iter)
+        self.app.register(
+            'GET', '/v1/a/src_cont/src_obj', swob.HTTPOk, {}, 'passed')
+        self.app.register(
+            'GET', '/v1/a/tgt_cont/tgt_obj', swob.HTTPOk,
+            {'x-timestamp': ts_now.normal,
+             'x-backend-timestamp': ts_now.internal,
+             # we'd expect last-modified to be ts_now as well, but we want the
+             # test to verify that x-timestamp is preferred when it is
+             # available so we deliberately set last-modified to another value
+             'last-modified': date_header_format(ts_then)},
+            'passed')
+        self.app.register(
+            'PUT', '/v1/a/ver_cont/007tgt_obj/%s' % ts_now.normal, swob.HTTPOk,
+            {}, None)
+        self.app.register(
+            'PUT', '/v1/a/tgt_cont/tgt_obj', swob.HTTPCreated, {}, 'passed')
+        cache = FakeCache({'sysmeta': {'versions-location': 'ver_cont'}})
+        req = Request.blank(
+            '/v1/a/src_cont/src_obj',
+            environ={'REQUEST_METHOD': 'COPY', 'swift.cache': cache,
+                     'CONTENT_LENGTH': '100'},
+            headers={'Destination': 'tgt_cont/tgt_obj'})
+        with mock_timestamp_now(ts_now):
+            status, headers, body = self.call_filter(req)
+        self.assertEqual(status, '201 Created')
+        self.assertEqual(len(self.authorized), 3)
+        self.assertEqual('GET', self.authorized[0].method)
+        self.assertEqual('/v1/a/src_cont/src_obj', self.authorized[0].path)
+        self.assertEqual('PUT', self.authorized[1].method)
+        self.assertEqual('/v1/a/tgt_cont/tgt_obj', self.authorized[1].path)
+        self.assertEqual(4, self.app.call_count)
+
     def test_copy_new_version_different_account(self):
         self.app.register(
             'GET', '/v1/src_a/src_cont/src_obj', swob.HTTPOk, {}, 'passed')
