@@ -14,11 +14,10 @@
 # limitations under the License.
 
 import unittest
-import mock
+from unittest import mock
 import tempfile
 import time
 
-from six.moves import range
 from test.unit import mock_check_drive
 
 from swift.common.swob import Request, HTTPException
@@ -498,11 +497,12 @@ class TestConstraints(unittest.TestCase):
                           constraints.valid_timestamp,
                           Request.blank('/', headers={
                               'X-Timestamp': 'asdf'}))
-        timestamp = utils.Timestamp.now()
+        timestamp = utils.Timestamp.now(offset=0xabc)
         req = Request.blank('/', headers={'X-Timestamp': timestamp.internal})
         self.assertEqual(timestamp, constraints.valid_timestamp(req))
         req = Request.blank('/', headers={'X-Timestamp': timestamp.normal})
-        self.assertEqual(timestamp, constraints.valid_timestamp(req))
+        self.assertEqual(utils.Timestamp(timestamp.normal),
+                         constraints.valid_timestamp(req))
 
     def test_check_utf8(self):
         unicode_sample = u'\uc77c\uc601'
