@@ -12,7 +12,6 @@
 # implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-import hashlib
 import os
 import shutil
 import tempfile
@@ -20,10 +19,11 @@ import unittest
 
 from swift.common import utils
 from swift.common.storage_policy import POLICIES
-from swift.common.utils import Timestamp
+from swift.common.utils import Timestamp, md5
+from test.unit import make_timestamp_iter
 
 
-def write_diskfile(df, timestamp, data='test data', frag_index=None,
+def write_diskfile(df, timestamp, data=b'test data', frag_index=None,
                    commit=True, legacy_durable=False, extra_metadata=None):
     # Helper method to write some data and metadata to a diskfile.
     # Optionally do not commit the diskfile, or commit but using a legacy
@@ -31,7 +31,7 @@ def write_diskfile(df, timestamp, data='test data', frag_index=None,
     with df.create() as writer:
         writer.write(data)
         metadata = {
-            'ETag': hashlib.md5(data).hexdigest(),
+            'ETag': md5(data, usedforsecurity=False).hexdigest(),
             'X-Timestamp': timestamp.internal,
             'Content-Length': str(len(data)),
         }
@@ -39,6 +39,7 @@ def write_diskfile(df, timestamp, data='test data', frag_index=None,
             metadata.update(extra_metadata)
         if frag_index is not None:
             metadata['X-Object-Sysmeta-Ec-Frag-Index'] = str(frag_index)
+            metadata['X-Object-Sysmeta-Ec-Etag'] = 'fake-etag'
         writer.put(metadata)
         if commit and legacy_durable:
             # simulate legacy .durable file creation
@@ -66,15 +67,22 @@ class BaseTest(unittest.TestCase):
         }
         # daemon will be set in subclass setUp
         self.daemon = None
+        self._ts_iter = make_timestamp_iter()
 
     def tearDown(self):
         shutil.rmtree(self.tmpdir, ignore_errors=True)
 
+    def ts(self):
+        """
+        Timestamps - forever.
+        """
+        return next(self._ts_iter)
+
     def _make_diskfile(self, device='dev', partition='9',
-                       account='a', container='c', obj='o', body='test',
+                       account='a', container='c', obj='o', body=b'test',
                        extra_metadata=None, policy=None,
                        frag_index=None, timestamp=None, df_mgr=None,
-                       commit=True, verify=True):
+                       commit=True, verify=True, **kwargs):
         policy = policy or POLICIES.legacy
         object_parts = account, container, obj
         timestamp = Timestamp.now() if timestamp is None else timestamp
@@ -82,14 +90,14 @@ class BaseTest(unittest.TestCase):
             df_mgr = self.daemon._df_router[policy]
         df = df_mgr.get_diskfile(
             device, partition, *object_parts, policy=policy,
-            frag_index=frag_index)
+            frag_index=frag_index, **kwargs)
         write_diskfile(df, timestamp, data=body, extra_metadata=extra_metadata,
                        commit=commit)
         if commit and verify:
             # when we write and commit stub data, sanity check it's readable
             # and not quarantined because of any validation check
             with df.open():
-                self.assertEqual(''.join(df.reader()), body)
+                self.assertEqual(b''.join(df.reader()), body)
             # sanity checks
             listing = os.listdir(df._datadir)
             self.assertTrue(listing)
@@ -98,11 +106,12 @@ class BaseTest(unittest.TestCase):
         return df
 
     def _make_open_diskfile(self, device='dev', partition='9',
-                            account='a', container='c', obj='o', body='test',
+                            account='a', container='c', obj='o', body=b'test',
                             extra_metadata=None, policy=None,
-                            frag_index=None, timestamp=None, df_mgr=None):
+                            frag_index=None, timestamp=None, df_mgr=None,
+                            commit=True, **kwargs):
         df = self._make_diskfile(device, partition, account, container, obj,
                                  body, extra_metadata, policy, frag_index,
-                                 timestamp, df_mgr)
+                                 timestamp, df_mgr, commit, **kwargs)
         df.open()
         return df
