@@ -15,7 +15,7 @@
 
 import unittest
 import json
-import mock
+from unittest import mock
 import os
 import signal
 import string
@@ -41,7 +41,8 @@ from swift.obj.diskfile import (
     get_auditor_status, HASH_FILE, HASH_INVALIDATIONS_FILE)
 from swift.common.exceptions import ClientException
 from swift.common.utils import (
-    mkdirs, normalize_timestamp, Timestamp, readconf, md5, PrefixLoggerAdapter)
+    mkdirs, normalize_timestamp, Timestamp, readconf, md5)
+from swift.common.utils.logs import SwiftLogAdapter
 from swift.common.storage_policy import (
     ECStoragePolicy, StoragePolicy, POLICIES, EC_POLICY)
 
@@ -169,6 +170,10 @@ class TestAuditorBase(BaseTestCase):
 @patch_policies(_mocked_policies)
 class TestAuditor(TestAuditorBase):
 
+    def setUp(self):
+        super().setUp()
+        self.ts = make_timestamp_iter()
+
     def test_worker_conf_parms(self):
         def check_common_defaults():
             self.assertEqual(auditor_worker.max_bytes_per_second, 10000000)
@@ -319,18 +324,18 @@ class TestAuditor(TestAuditorBase):
                                                self.rcache, self.devices)
         data = b'0' * 1024
         etag = md5(usedforsecurity=False)
-        timestamp = str(normalize_timestamp(time.time()))
+        timestamp = next(self.ts)
         with self.disk_file.create() as writer:
             writer.write(data)
             etag.update(data)
             etag = etag.hexdigest()
             metadata = {
                 'ETag': etag,
-                'X-Timestamp': timestamp,
+                'X-Timestamp': timestamp.normal,
                 'Content-Length': str(os.fstat(writer._fd).st_size),
             }
             writer.put(metadata)
-            writer.commit(Timestamp(timestamp))
+            writer.commit(timestamp)
             pre_quarantines = auditor_worker.quarantines
 
         # remake so it will have metadata
@@ -342,12 +347,14 @@ class TestAuditor(TestAuditorBase):
                           policy=POLICIES.legacy))
         self.assertEqual(auditor_worker.quarantines, pre_quarantines)
         etag = md5(b'1' + b'0' * 1023, usedforsecurity=False).hexdigest()
+        ts2 = next(self.ts)
         metadata['ETag'] = etag
+        metadata['X-Timestamp'] = ts2.normal
 
         with self.disk_file.create() as writer:
             writer.write(data)
             writer.put(metadata)
-            writer.commit(Timestamp(timestamp))
+            writer.commit(ts2)
 
         auditor_worker.object_audit(
             AuditLocation(self.disk_file._datadir, 'sda', '0',
@@ -670,7 +677,7 @@ class TestAuditor(TestAuditorBase):
         auditor_worker = auditor.AuditorWorker(self.conf, self.logger,
                                                self.rcache, self.devices)
         auditor_worker.log_time = 0
-        timestamp = str(normalize_timestamp(time.time()))
+        timestamp = next(self.ts)
         pre_quarantines = auditor_worker.quarantines
         data = b'0' * 1024
 
@@ -679,11 +686,11 @@ class TestAuditor(TestAuditorBase):
                 writer.write(data)
                 metadata = {
                     'ETag': md5(data, usedforsecurity=False).hexdigest(),
-                    'X-Timestamp': timestamp,
+                    'X-Timestamp': timestamp.internal,
                     'Content-Length': str(os.fstat(writer._fd).st_size),
                 }
                 writer.put(metadata)
-                writer.commit(Timestamp(timestamp))
+                writer.commit(timestamp)
 
         # policy 0
         write_file(self.disk_file)
@@ -699,17 +706,18 @@ class TestAuditor(TestAuditorBase):
         self.assertEqual(auditor_worker.stats_buckets[10240], 0)
 
         # pick up some additional code coverage, large file
+        ts2 = next(self.ts)
         data = b'0' * 1024 * 1024
         for df in (self.disk_file, self.disk_file_ec):
             with df.create() as writer:
                 writer.write(data)
                 metadata = {
                     'ETag': md5(data, usedforsecurity=False).hexdigest(),
-                    'X-Timestamp': timestamp,
+                    'X-Timestamp': ts2.internal,
                     'Content-Length': str(os.fstat(writer._fd).st_size),
                 }
                 writer.put(metadata)
-                writer.commit(Timestamp(timestamp))
+                writer.commit(ts2)
         auditor_worker.audit_all_objects(device_dirs=['sda', 'sdb'])
         self.assertEqual(auditor_worker.quarantines, pre_quarantines)
         # still have the 1024 byte object left in policy-1 (plus the
@@ -1696,8 +1704,13 @@ class TestAuditWatchers(TestAuditorBase):
         self.assertEqual(len(calls), 6)
 
         self.assertEqual(calls[0], ["__init__", conf, mock.ANY])
-        self.assertIsInstance(calls[0][2], PrefixLoggerAdapter)
-        self.assertIs(calls[0][2].logger, self.logger)
+        watcher_logger = calls[0][2]
+        self.assertIsInstance(calls[0][2], SwiftLogAdapter)
+        self.assertIs(calls[0][2].logger, self.logger.logger)
+        self.assertEqual(watcher_logger.server, my_auditor.logger.server)
+        self.assertEqual('', my_auditor.logger.prefix)  # sanity check
+        self.assertEqual('[audit-watcher test_watcher1] ',
+                         watcher_logger.prefix)
 
         self.assertEqual(calls[1], ["start", "ZBF"])
 
@@ -1731,10 +1744,15 @@ class TestAuditWatchers(TestAuditorBase):
 
         self.assertEqual(calls[5], ["end"])
 
+        # check that watcher has not mutated the prefix of the auditor logger
+        my_auditor.logger.debug('the auditor has no logger prefix')
         log_lines = self.logger.get_lines_for_level('debug')
         self.assertIn(
             "[audit-watcher test_watcher1] getting started",
             log_lines)
+        self.assertIn(
+            "the auditor has no logger prefix",
+            log_lines)
 
     def test_builtin_watchers(self):
 
